#------------------------------------------------------------------
# $Copyright: Copyright (c) 2011 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
# $Header: /project/isv-agent/VProAgents/WebLogic/WebLogic.pm,v 1.19 2011/06/14 11:06:31 lpatil Exp $
#
# $Author: lpatil $
# $Date: 2011/06/14 11:06:31 $
# $Id: WebLogic.pm,v 1.19 2011/06/14 11:06:31 lpatil Exp $
# $Revision: 1.19 $
#
# Provides:                            WebLogic
# Modification History:
# Known bugs/caveats:
#------------------------------------------------------------------

#------------
# Be strict..
#------------
use strict;
use Comms;

#----------------------
# Declare the package..
#----------------------
package WebLogic;

use IO::File;
use IO::Handle;
use File::Basename;

#----------------------------------------------------
# Hard code if you still didn't get it from the env..
#----------------------------------------------------
my $VCS_HOME ||= $ENV{VCS_HOME} ||= "/opt/VRTSvcs";

#-------------------------------------
# Include required framework modules..
#-------------------------------------
use Sys;            # support generalized UNIX commands
use Proc;           # For calls to the process routines
use Version;        # Version display generator
use VCSagentFW;     # Primary Agent Interface to the ACC Library
use VCSDefines;     # This contains constants that VCS uses
use POSIX qw(tmpnam);
use POSIX qw ( uname );
use ag_i18n_inc;

#--------------------------------------------------------
# Setup the global log array for generating trace dump.
#--------------------------------------------------------
@::gsLogString                        = ();
%::hWebLogicTypeValues               = ();

my $bEnv = 0;

use constant EPOP                     =>  3;
use constant DONT_SOURCE_USER_ENV     =>  0;
use constant CHECK_INTERVAL           =>  5;
use constant MAXBUFFERSIZE            => 4096;

#--------------------------------
# Define WebLogic subroutines..
#--------------------------------

=pod

 -------------------------------------------------------------------------
 Function :  bool ArgsValid ( IN String EntryPointName,
                              IN String BEA_HOME,
                              IN String WlstScript,
                              IN String DomainName,
                              IN String DomainDir,   
                              IN String ListenAddressPort,
                              IN String MonitorProgram,
                              IN String nmListenAddressPort,
                              IN String nmType,
                              IN String nmHome,
                              IN String ServerName,
                              IN String ServerRole,
                              IN String User,
                              IN String WLSUser,
                              IN String WLSPassword,
                              IN String nmUser,
                              IN String nmPassword,                              
                              IN Integer ShutdownTimeout,                              
                              IN String ServerStartProgram,
                              IN String ServerStopProgram,
                              IN Boolean RequireAdminServer,
                              IN String SecondLevelMonitor );

 Description : Validates the ArgList passed to the entry point.

 Parameters :

 $sEntryPointName             Entry point name
 $localBEA_HOME               BEA Home Directory
 $localWlstScript             WLST Script Location
 $localDomainName             WebLogic Domain Name
 $localDomainDir              WebLogic Domain Directory   
 $localListenAddressPort      Listen Address and Port of this WLS Server
 $localMonitorProgram         External Monitor Program
 $localnmListenAddressPort    Listen Address and Port of this node manager
 $localnmType                 Node Manager Type
 $localnmHome                 Node Manager Home Directory
 $localServerName             WLS Server Name
 $localServerRole             WLS Server Role
 $localUser                   Unix User
 $localWLSUser                WebLogic Server User
 $localWLSPassword            WebLogic Server Password
 $localnmUser                Nodemanager User (shared secret)
 $localnmPassword            Nodemanager Password (shared secret)
 $localShutdownTimeout        Time that will be passed to shutdown command
 $localServerStartProgram     WebLogic Server Start program
 $localServerStopProgram      WebLogic Server Stop program
 $localRequireAdminServer     Boolean Require Admin Server flag
 $localAdminServerMaxWait     Max sec. to wait till AdminServer become available
 $localSecondLevelMonitor     Second Level Monitor
 $localnmUser     Nodemanager User
 $localnmPassword     Second Level Monitor Password
 
 Returns : boolean value in the set of {0,1} where
           0 false
           1 true

 -------------------------------------------------------------------------

=cut

sub ArgsValid {
  #----------------------------------------
  # Set up local vars from passed in ones..
  #----------------------------------------
  my ( $sEntryPointName,
       $localAdminURL,
       $localBEA_HOME,
       $localWlstScript,
       $localDomainName,
       $localDomainDir,
       $localListenAddressPort,
       $localMonitorProgram,
       $localnmListenAddressPort,
       $localnmType,
       $localnmHome,
       $localServerName,
       $localServerRole,
       $localUser,
       $localWLSUser,
       $localWLSPassword,
       $localnmUser,
       $localnmPassword,       
       $localServerStartProgram,
       $localServerStopProgram,
       $localShutdownTimeout,
       $localRequireAdminServer,
       $localAdminServerMaxWait,
       $localSecondLevelMonitor) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20001, "Subroutine <%s> called with:
  EntryPointName           [%s]
  AdminURL                 [%s]
  BEA_HOME                 [%s]
  WlstScript               [%s]
  DomainName               [%s]
  DomainDir                [%s]
  ListenAddressPort        [%s]
  MonitorProgram           [%s]
  nmListenAddressPort      [%s]
  nmType                   [%s]
  nmHome                   [%s]
  ServerName               [%s]
  ServerRole               [%s]
  User                     [%s]
  WLSUser                  [%s]
  WLSPassword              [%s]
  nmUser                   [%s]
  nmPassword 		   [%s]
  ServerStartProgram       [%s]
  ServerStopProgram	   [%s]
  ShutdownTimeout          [%s]
  RequireAdminServer       [%s]
  AdminServerMaxWait       [%s]
  SecondLevelMonitor       [%s]"
,
  $sMyName,  $sEntryPointName, $localAdminURL, $localBEA_HOME, $localWlstScript, $localDomainName,
  $localDomainDir,$localListenAddressPort,  $localMonitorProgram, $localnmListenAddressPort,
  $localnmType, $localnmHome, $localServerName, $localServerRole, $localUser,  $localWLSUser,
  $localWLSPassword, $localnmUser, $localnmPassword, $localServerStartProgram, 
  $localServerStopProgram, $localShutdownTimeout, $localRequireAdminServer, $localAdminServerMaxWait, $localSecondLevelMonitor );

  my $iTimeNow = time;
  &logTrace( 20002, "<%s>:time is [%s]",$sMyName, $iTimeNow);

  #-------------------------------------
  # Set return code to hopeful success..
  #-------------------------------------
  my $iReturnCode = 1;

  #-------------------------------------------
  # Validate ServerRole for all entry points.
  #-------------------------------------------
  if ( "Administrative" ne $localServerRole && 
       "Managed"        ne $localServerRole && 
       "NodeManager"    ne $localServerRole)
  {
    #-----------------------
    # Unsuppored ServerRole
    #-----------------------
    &logError( 20031, "<%s>:Invalid ServerRole [%s] defined for Server [%s]. Allowed values are [Administrative,Managed,NodeManager]",
                       $sMyName,$localServerRole,$localServerName);
    return 0;   # We need ServerRole first, then only we can check other attributes.
  }

  if ( $localServerStartProgram && $localServerStopProgram )
  {
    $::gbNMBased = 0;
    &logTrace( 20003, "<%s>:This is non-NodeManaged based configuration.",$sMyName);
  }
  elsif ( $localServerStartProgram || $localServerStopProgram )
  {
    &logError( 20004, "<%s>:Unsupported configuration. Both ServerStartProgram and ServerStopProgram should either be specified or not specified.",$sMyName);
    $iReturnCode = 0;
  }
  else
  {
    &logTrace( 20005, "<%s>:This is NodeManager based configuration.",$sMyName);
  }

  #--------------------------------------------
  # Create hash to weed out empty args values..
  #--------------------------------------------
  my ( @lsMissing, %hsArgList ) = ();
  my @lsArgs =
  qw ( EntryPointName AdminURL BEA_HOME WlstScript DomainName DomainDir ListenAddressPort
       MonitorProgram nmListenAddressPort nmType nmHome ServerName  ServerRole  User WLSUser WLSPassword 
       nmUser nmPassword ServerStartProgram ServerStopProgram ShutdownTimeout RequireAdminServer 
       AdminServerMaxWait SecondLevelMonitor );

  @hsArgList { @lsArgs } = @_; # Assign..

  #----------------------------------------
  # Reqd. attributes based on entry point..
  #----------------------------------------
  my @lsOnlineArgs        = qw (BEA_HOME WlstScript nmType User WLSUser WLSPassword nmUser nmPassword);
  my @lsOfflineArgs       = qw (BEA_HOME WlstScript nmType User WLSUser WLSPassword nmUser nmPassword);
  my @lsMonitorArgs       = qw (BEA_HOME WlstScript nmType User WLSUser WLSPassword nmUser nmPassword);
  my @lsCleanArgs         = qw (BEA_HOME WlstScript nmType User WLSUser WLSPassword nmUser nmPassword);
  my @lsImfRegisterArgs   = qw (BEA_HOME WlstScript nmType User ServerName);

  if ( $::gbNMBased == 0 )
  {
    #-------------------------------------------
    # Non NodeManager based configuration.
    #-------------------------------------------
 
    if ( $localServerRole eq "NodeManager" ) 
    {
       &logError( 20006, "<%s>:ServerRole cannot be [NodeManager] in Non NodeManager Based Configurations",
                          $sMyName);
       $iReturnCode = 0;
    }

    if ( $localSecondLevelMonitor ge 1 )
    { 
       @lsOnlineArgs  = qw (BEA_HOME WlstScript User WLSUser WLSPassword);
       @lsOfflineArgs = qw (BEA_HOME WlstScript User WLSUser WLSPassword);
       @lsMonitorArgs = qw (BEA_HOME WlstScript User WLSUser WLSPassword);
    }
    else
    {
       @lsOnlineArgs  = qw (BEA_HOME User);
       @lsOfflineArgs = qw (BEA_HOME User);
       @lsMonitorArgs = qw (BEA_HOME User);
    }
    @lsCleanArgs   = qw (BEA_HOME User);
    @lsImfRegisterArgs   = qw (BEA_HOME User ServerName );
  }
  else {
	if ( $localServerRole eq "NodeManager" )
	{
	   @lsOnlineArgs        = qw (BEA_HOME WlstScript nmType User nmHome );
	   @lsOfflineArgs       = qw (BEA_HOME WlstScript nmType User );
	   if ( $localSecondLevelMonitor == 0 )
	   {
		  @lsMonitorArgs       = qw (BEA_HOME WlstScript nmType User );
	   }
	   else
	   {
		  @lsMonitorArgs       = qw (BEA_HOME WlstScript nmType User nmUser nmPassword);
	   }
	}
  }

  my %hlsNeeded = ( 'online'        => \@lsOnlineArgs,
                    'offline'       => \@lsOfflineArgs,
                    'monitor'       => \@lsMonitorArgs,
                    'clean'         => \@lsCleanArgs,
                    'imf_register'	=> \@lsImfRegisterArgs );

  my @lsReqdArgs = @{$hlsNeeded { $sEntryPointName }};

  #-----------------------
  # Check what's missing..
  #-----------------------
  if ( @lsMissing = grep { $hsArgList { $_ } !~ /\S/o } @lsReqdArgs ) {
    &logError ( 20007,
    "<%s> The following mandatory fields were not set [%s], Total [%s]",
    $sMyName, join ( ' ', @lsMissing ), scalar @lsMissing );
    $iReturnCode = 0; # Failure..
  }

  $iReturnCode || return $iReturnCode; # Return, if failed..

  #---------------------------------------------------------------
  # Validate AdminURL , if defined in case of ServerRole Managed.
  #---------------------------------------------------------------
  if ( $sEntryPointName eq "online" || $sEntryPointName eq "monitor" )
  {
    if ( $localServerRole eq "Managed" )
    {
      my @lsURLcomponents =();
      if( ( defined $localAdminURL ) && ( $localAdminURL ne "" ) )
      {
        @lsURLcomponents = ParseURL( $localAdminURL );
        if( !defined $lsURLcomponents[1] )
        {
          &logError( 20008,"<%s>:Invalid format for Admin URL[%s]",
                            $sMyName, $localAdminURL);
          $iReturnCode = 0;
        }
        else
        {
          if ( $sEntryPointName eq "online") {
            my $address = gethostbyname( $lsURLcomponents[1] );
            if( !defined( $address ) )
            {
              &logError(20009,
              "<%s>:Invalid name for AdminURL. Host [%s] not found.",
              $sMyName,$lsURLcomponents[0] );
              $iReturnCode = 0;
            }
          }
        }         
      }
      else
      {
          &logError( 20010, "<%s>:Invalid format for Admin URL [%s]",
                             $sMyName,$localAdminURL );
          $iReturnCode = 0;
      }
    }
  }

  #----------------------------------------------------------------
  # BEA_HOME should be accessible for online & offline entrypoints.
  #----------------------------------------------------------------
  if ( $sEntryPointName eq "online" 
    || $sEntryPointName eq "offline" )
  {
    if ( ! -d $localBEA_HOME )
    {
      &logError( 20011, "<%s>:BEA_HOME [%s] for this server not accessible.",
                         $sMyName,$localBEA_HOME);
      $iReturnCode = 0;
    }
  }
  
  #---------------------------------------------
  # WlstScript should be accessible if provided
  #---------------------------------------------
  if ( ( $sEntryPointName eq "online" 
    || $sEntryPointName eq "offline" ) && $localWlstScript )
  {
    if ( ! -f $localWlstScript )
    {
      &logError( 20012, "<%s>:WlstScript [%s] for this server not accessible.",
                         $sMyName,$localWlstScript);
      $iReturnCode = 0;
    }
  }

  #-------------------------------------------------------------------------------
  # DomainDir should not be NULL for Administrative or Managed Role
  # DomainDir should not be NULL for Node Manager if SecondLevelMonitor is enabled
  #-------------------------------------------------------------------------------
  if ( ($localServerRole eq "Managed" || $localServerRole eq "Administrative") 
        && ! $localDomainDir )
  {
    &logError( 20013,"<%s>:DomainDir not defined for Server [%s]",
                      $sMyName,$localServerName);
    $iReturnCode = 0;
  }

  if ( ($localServerRole eq "NodeManager") && ($localSecondLevelMonitor ge 1)  && ! $localDomainDir )
  {
    &logError( 20014,"<%s>:DomainDir not defined, it is required for the SecondLevelMonitor.",
                      $sMyName);
    $iReturnCode = 0;
  }


  #-----------------------------------------------------------------------------------------
  # DomainDir should be accessible for online entrypoint for Administrative or Managed Role.
  #-----------------------------------------------------------------------------------------
  if ( $sEntryPointName eq "online")
  {
    if ($localServerRole eq "Managed" || $localServerRole eq "Administrative") 
    { 
       if ( ! -d $localDomainDir )
       {
          &logError( 20015, "<%s>:DomainDir [%s] for this server not accessible.",
                          $sMyName,$localDomainDir);
          $iReturnCode = 0;
       }
    }
  }
  
  #--------------------------------------------------------------------------------
  # DomainName should not be NULL for Administrative or Managed Role
  # DomainName should not be NULL for Node Manager if SecondLevelMonitor is enabled
  #--------------------------------------------------------------------------------
  if ( ($localServerRole eq "Managed" || $localServerRole eq "Administrative") 
        && ! $localDomainName )
  {
    &logError( 20016,"<%s>:DomainName not defined for Server [%s]",
                      $sMyName,$localServerName);
    $iReturnCode = 0;
  }

  if ( ($localServerRole eq "NodeManager") && ($localSecondLevelMonitor ge 1)  && ! $localDomainName )
  {
     &logError( 20017,"<%s>:DomainName not defined, it is required for the SecondLevelMonitor.",
                      $sMyName);
     $iReturnCode = 0;
  }

  #-------------------------------------------------------------------------
  # Validate ListenAddressPort if Server Role is Administrative or Managed
  #-------------------------------------------------------------------------
  if ( $localServerRole eq "Managed" || $localServerRole eq "Administrative" )
  {
    if ( $localListenAddressPort )
    {
      my (@sParsed) = $localListenAddressPort =~ m@^([\w\.\-]+):(\d+)$@;
      if ( ! defined($sParsed[1]) )
      {
        &logError( 20018,
        "<%s>:Invalid format of ListenAddressPort [%s] defined for Server [%s]",
         $sMyName, $localListenAddressPort, $localServerName);
        $iReturnCode = 0;
      }
      else
      {
        if ( $sEntryPointName eq "online") {
          my $address = gethostbyname( $sParsed[0] );
          if( !defined( $address ) )
          {
            &logError(20019,"<%s>:Invalid ListenAddress. Host [%s] not found",
                             $sMyName, $sParsed[0] );
            $iReturnCode = 0;
          }
        }
      }
    }
    else
    {
      &logError( 20020,"<%s>:No ListenAddressPort defined for Server [%s]",
                        $sMyName, $localServerName);
      $iReturnCode = 0;
    }
  }

  #-----------------------------------------------------------------
  # In online check whether MonitorProgram exists and is executable
  #-----------------------------------------------------------------
  my ($sMonitorProgram,$sMonitorProgramArgs) = 
                           split(/\s/,$localMonitorProgram,2);
  if ( "online" eq $sEntryPointName && 
     ( $sMonitorProgram && 
    !( -x $sMonitorProgram ) ) )
  {
    &logError( 20021,"<%s>:MonitorProgram [%s] not executable.",
                      $sMyName, $sMonitorProgram);
    $iReturnCode = 0;
  }

  #-------------------------------
  # Validate nmListenAddressPort
  #-------------------------------
  if ( $::gbNMBased )
  {
    if ( $localnmListenAddressPort )
    {
      my (@sParsed) = $localnmListenAddressPort =~ m@^([\w\.\-]+):(\d+)$@;
      if ( !defined($sParsed[1]))
      {
        &logError( 20022,"<%s>:Invalid format of nmListenAddressPort [%s] defined for [%s] Server [%s]", 
        $sMyName, $localnmListenAddressPort,$localServerRole,$localServerName);
        $iReturnCode = 0;
      }
      else
      {
        if ( $sEntryPointName eq "online") {
          my $address = gethostbyname( $sParsed[0] );
          if( !defined( $address ) )
          {
            &logError( 20023, "<%s>:Invalid nmListenAddress. Host [%s] not found",
                               $sMyName,$sParsed[0] );
            $iReturnCode = 0;
          }
        }
      }
    }
    else
    {
      if ( "NodeManager" ne $localServerRole )
      {
        &logError( 20024,"<%s>:No nmListenAddressPort defined for Server [%s]",
                          $sMyName, $localServerName);
      }
      else
      {
        &logError( 20025,"<%s>:No nmListenAddressPort defined for this NodeManager", $sMyName)
      }
      $iReturnCode = 0;
    }
  }
  else
  {
    if ( $localnmListenAddressPort )
    {
        &logError( 20026,"<%s>:nmListenAddressPort should not be defined for NON NodeManager based Servers.", $sMyName);
        $iReturnCode = 0;
    }
  }


  if ( $::gbNMBased )
  {
    #-----------------------------------
    # Validate nmType 
    #-----------------------------------
    if ( "online"  eq $sEntryPointName || "offline" eq $sEntryPointName || 
       ( "monitor" eq $sEntryPointName && "NodeManager" eq $localServerRole) )
    {
      if ( ($localnmType ne "ssl") && ($localnmType ne "plain") && 
           ($localnmType ne "rsh") && ($localnmType ne "ssh") )
      {
        #------------------------------
        # Unsuppored Node Manager Type.
        #------------------------------
        if ( "NodeManager" ne $localServerRole )
        {
          &logError( 20027, "<%s>:Invalid nmType [%s] defined for Server [%s]. Allowed values are [plain,rsh,ssh,ssl]",
                             $sMyName, $localnmType, $localServerName);
        }
        else
        {
          &logError( 20028, "<%s>:Invalid nmType [%s] defined for Nodemanager. Allowed values are [plain,rsh,ssh,ssl]",
                             $sMyName, $localnmType);
        }
        $iReturnCode = 0;
      }
    }
    #-----------------------------------
    # Validate nmHome 
    #-----------------------------------  
	if ( $sEntryPointName eq "online" && "NodeManager" eq $localServerRole ) 
	{
		if ( ! $localnmHome ) 
		{  
			&logError( 20309, "<%s>:nmHome is not defined for NodeManager.", 
				$sMyName);
			$iReturnCode = 0;
		}
		else 
		{
			if ( ! -d $localnmHome )
			{
				&logError( 20029, "<%s>:nmHome [%s] for this server not accessible.",
					$sMyName,$localnmHome);
				$iReturnCode = 0;
			}
		}
	}
  }  

  #----------------------------------------------------------------
  # Validate ServerName for all entry points for Administrative 
  # and Managed ServerRole
  #----------------------------------------------------------------
  if ( ! $localServerName  && ("Administrative" eq $localServerRole || "Managed" eq $localServerRole))
  {
    #-------------------------------
    # ServerName should be defined.
    #-------------------------------
    &logError( 20030, "<%s>:ServerName not defined.",$sMyName);
    $iReturnCode = 0;
  }


  if ( "Administrative" eq $localServerRole )
  {
     # Check for Env Flag for different OS

     my $sOSVer = undef;
     #--------------------------------------------------
     # Attempt to get environment of running processes..
     #--------------------------------------------------
     if ( $^O eq 'solaris' ) {
        $sOSVer = ( uname ) [2]; # Get OS version..
 
        #--------------------------------
        # Handle Solaris 10 local zones..
        #--------------------------------
        if ( $sOSVer eq '5.10' && # Solaris 10..
           ! kill ( 0, 1 ) ) {  # Not in Global zone..
           $bEnv = 0;
     
           &logTrace ( 20032,
           "<%s> Running inside OS [%s] OSVer [%s] inside local zone",
           $sMyName, $^O, $sOSVer );
        }
        else {
          ++ $bEnv; # Set global..

          &logTrace ( 20033,
          "<%s> Running inside OS [%s] OSVer [%s]. Env flag set to [%s]",
          $sMyName, $^O, $sOSVer, $bEnv );
       }
     }
     elsif ( $^O eq 'hpux' ) {
        $bEnv = 0;
     }
     else {
        ++ $bEnv;
     }

     &logTrace ( 20034,
     "<%s> Environment flag set to [%s] on OS [%s]",
     $sMyName, $bEnv, $^O );
  }

  if ( $sEntryPointName ne "clean" )
  {
    #---------------------------------------------
    # Check whether the User is valid on system.
    #---------------------------------------------
    my $spwname = getpwnam($localUser);
    if ( !defined($spwname) )
    {
      &logError(20035, "<%s>:User name [%s] invalid.",$sMyName,$localUser);
      $iReturnCode = 0 ;
    }
  }

  $iReturnCode ||
  &logWarn ( 20036,
  "<%s> Some of the attributes could not be validated. Returning [%s]",
  $sMyName, $iReturnCode );

  return $iReturnCode ;
}

=pod

 -------------------------------------------------------------------------
 Function : bool Start ( IN Integer State,
                         IN Integer IState,
                         IN String  BEA_HOME,
                         IN String  WlstScript, 
                         IN String  User,
                         IN String  WLSUser,
                         IN String  WLSPassword,
                         IN String  nmUser,
                         IN String  nmPassword,                          
                         IN String  nmType,
                         IN String  nmHome,
                         IN String  ListenAddressPort,
                         IN String  nmListenAddressPort,
                         IN String  ServerName,
                         IN String  ServerRole,
                         IN String  AdminURL
                         IN String  DomainName,
                         IN String  DomainDir, 
                         IN Integer SecondLevelMonitor,               
                         IN String  MonitorProgram,
                         IN Boolean RequireAdminServer,
                         IN Integer AdminServerMaxWait
                         IN Integer ExpirationTime
                        );

 Description : Start logic needed for online entry pt

 Parameters :

  $sBEA_HOME              BEA Home Directory
  $sWlstScript            WLST Script Location
  $sUser                  Unix User
  $sWLSUser               WebLogic Server User
  $sWLSPassword           WebLogic Server Password
  $snmUser               Nodemanager User (shared secret)
  $snmPassword           Nodemanager Password  (shared secret)  
  $snmType                Node Manager Type
  $snmHome				  Node Manager Home
  $sListenAddressPort     ListenAddress and Port of this WLS Server
  $snmListenAddressPort   ListenAddress and Port of this Node manager
  $sServerName            WLS Server Name
  $sServerRole            WLS Server Role
  $sAdminURL              Administrative Server's URL
  $sDomainName            WebLogic Domain Name
  $sDomainDir             WLS Domain Directory  
  $sSecondLevelMonitor    Second Level Monitor
  $sMonitorProgram        External Monitor Program
  $bRequireAdminServer    Boolean Require Admin Server flag
  $iAdminServerMaxWait    Max seconds to wait till AdminServer becomes available
  $iExpirationTime        Expiration Time

 
 Returns : boolean value in the set of {0,1} where
           0 false
           1 true

 -------------------------------------------------------------------------

=cut

sub Start {
  #----------------
  # Get Arguments..
  #----------------
  my ( $iState,
       $iIState,
       $sBEA_HOME,
       $sWlstScript, 
       $sUser,
       $sWLSUser,
       $sWLSPassword,
       $snmUser,
       $snmPassword,       
       $snmType,
       $snmHome,
       $sListenAddressPort,
       $snmListenAddressPort,
       $sServerName,
       $sServerRole,
       $sAdminURL,
       $sDomainName,
       $sDomainDir,
       $iSecondLevelMonitor,
       $sServerStartProgram,
       $sMonitorProgram,
       $bRequireAdminServer,
       $iAdminServerMaxWait,
       $iExpirationTime  )    = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace( 20037, "Subroutine <%s> called with: 
  State                    [%s]
  iState                   [%s]
  BEA_HOME                 [%s]
  WlstScript               [%s]
  User                     [%s]
  WLSUser                  [%s]
  WLSPassword              [%s]
  nmUser   		   [%s]
  nmPassword 		   [%s]  
  nmType                   [%s]
  nmHome                   [%s]
  ListenAddressPort        [%s]
  nmListenAddressPort      [%s]
  ServerName               [%s]
  ServerRole               [%s]
  AdminURL                 [%s]
  DomainName               [%s]
  DomainDir                [%s]
  SecondLevelMonitor       [%s]
  ServerStartProgram       [%s]
  MonitorProgram           [%s]
  RequireAdminServer       [%s]
  AdminServerMaxWait       [%s]
  ExpirationTime           [%s]",
  $sMyName, $iState, $iIState, $sBEA_HOME, $sWlstScript, $sUser, $sWLSUser, $sWLSPassword, $snmUser, $snmPassword,
  $snmType, $snmHome, $sListenAddressPort, $snmListenAddressPort, $sServerName, 
  $sServerRole, $sAdminURL, $sDomainName, $sDomainDir, $iSecondLevelMonitor, $sServerStartProgram,
  $sMonitorProgram, $bRequireAdminServer, $iAdminServerMaxWait, $iExpirationTime  );

  #--------------------------------------
  # Set Return Code Optimistically to 1..
  #--------------------------------------
  my $iReturnCode = 1;

  my ($sHost,$sPort)     = split(/:/,$sListenAddressPort);
  my ($snmHost,$snmPort) = split(/:/,$snmListenAddressPort);

  my $sResourceType   = $ENV{'VCS_LOG_AGENT_NAME'};
  my $sResourceName   = $ENV{'VCS_LOG_RESOURCE_NAME'};
  my $sEntryPointName = $ENV{'VCS_LOG_SCRIPT_NAME'};

  #-----------------------------------------------------------------------
  # Remove CleanHasJustRunIndicatorFile just in case there was left over
  # errantly , somehow...
  #-----------------------------------------------------------------------
  unlink ( "/tmp/$sResourceType.$sResourceName.CleanHasJustrun");
  
  my $sCommand       = $sWlstScript;
  my $sCommandArgs;
  my $sEnvFile       = "";
  my $sWLSOutFile    = "/tmp/.VRTS$sResourceType/$sResourceName"."_$sEntryPointName.out";
  my $sStopString;
  my @sCommandArgsList;
  my $sCommandArgsForDisplay;
  my $bTimedOut;
  my $iTimeOut;
  my $iExitCode;
  my @lsCommandOut;

  my ( $sName      ,
       $sPassword  ,
       $iUid       ,
       $iGid       ,
       $iQuota     ,
       $sComment   ,
       $sGcos      ,
       $sHome      ,
       $sUserShell ,
       $sExpire    ) = getpwnam($sUser);

  #---------------------------------------------------------------------
  # Form the command line string for output redirection depending upon
  # the user shell.
  #---------------------------------------------------------------------
  my $bCshSet         = 0;

  if (! -d "/tmp/.VRTS$sResourceType" )
  {
    &logTrace(20038,"<%s>: Creating directory /tmp/.VRTS$sResourceType",$sMyName);
    mkdir ("/tmp/.VRTS$sResourceType", 0777);
    chmod (0777, "/tmp/.VRTS$sResourceType");
  }

  if ( "NodeManager" eq $sServerRole )
  {
    @sCommandArgsList = (
                        "startNodeManager\(verbose=\"true\",NodeManagerHome=\"$snmHome\",ListenPort=\"$snmPort\",ListenAddress=\"$snmHost\"\)\n",
                        "exit\(\)\n"
                        );

$sCommandArgsForDisplay = " << _EOF_
startNodeManager\(verbose=\\\"true\\\",NodeManagerHome=\\\"$snmHome\\\",ListenPort=\\\"$snmPort\\\",ListenAddress=\\\"$snmHost\\\"\)
exit()
_EOF_
";

  }
  else
  {
    if ( $::gbNMBased )
    {

      #-------------------------------------------------------------
      # Form the wlst command for Administrative/Managed ServerRole.
      #-------------------------------------------------------------

      my $sDecryptedPassword = DecryptPassword($snmPassword);
      if  ( ! $sDecryptedPassword )
      {
        &logError(20039,"<%s>:Could not decrypt the password",$sMyName);
         return 0;
      }

      @sCommandArgsList = (
                         "nmConnect\(\"$snmUser\",\"$sDecryptedPassword\",\"$snmHost\",\"$snmPort\",\"$sDomainName\",\"$sDomainDir\",\"$snmType\"\)\n",
                         "nmStart\(\"$sServerName\"\)\n",
                         "exit\(\)\n"
                       );

$sCommandArgsForDisplay = " << _EOF_
nmConnect(\"$snmUser\",XXXXXX,\"$snmHost\",\"$snmPort\",\"$sDomainName\",\"$sDomainDir\",\"$snmType\");
nmStart(\"$sServerName\")
exit()
_EOF_
";
    }
  }
    &logTrace( 20040, "<%s>:Command Arguments [%s]",
                                  $sMyName , $sCommandArgsForDisplay);

    if ( "Managed" eq $sServerRole && $bRequireAdminServer )
    {
      #---------------------------------------------------------------------
      # ServerRole is Managed and RequireAdminServer is TRUE.
      # Cannot start this Managed Server till the Administrative server
      # responds at AdminURL.
      #---------------------------------------------------------------------

      if (WebLogic::WaitForAdminServer($iExpirationTime,$sAdminURL) )
      {
        &logTrace(20041,"<%s>:AdminServer is available at [%s].",
                                     $sMyName, $sAdminURL);
      }
      else
      {
         #----------------------------------------------------------------
         # Administrative Server not available , abandon online attempts.
         #----------------------------------------------------------------

         &logWarn(20042,"<%s>:AdminServer at URL [%s] did not respond correctly before the OnlineTimeout was encountered, abandoning online attempt as the RequireAdminServer attribute is set", $sMyName, $sAdminURL);
        return 0;
      }
    }
    elsif ( "Managed" eq $sServerRole && $iAdminServerMaxWait )
    {
      #---------------------------------------------------------------------
      # Wait till AdminServerMaxWait seconds for a response from
      # Administrative server's AdminURL. As soon as Administrative server
      # responds on AdminURL go ahead and start this Managed Server.
      #---------------------------------------------------------------------

      my $iMaxWait = time + $iAdminServerMaxWait;
      WebLogic::WaitForAdminServer($iMaxWait,$sAdminURL);
    }

    $iTimeOut = $iExpirationTime - time;
    if ( 5 > $iTimeOut )
    {
      &logWarn(20043,"<%s>:Not enough time left [%s seconds] to start the online procedures.",$sMyName,$iTimeOut);
    }



    #------------------------
    # Remove the output file.
    #------------------------
    unlink ("$sWLSOutFile");

    #------------------------------------
    # Execute the actual start command.
    #------------------------------------

    if ( $::gbNMBased )
    {
      $iReturnCode = RunCmdViaPipe ($sUser,
                                    $sCommand,
                                    $sWLSOutFile,
                                    \@sCommandArgsList,
                                    $sBEA_HOME,
                                    $sWlstScript,
                                    \$iExitCode,
                                    $iTimeOut,
                                    \$bTimedOut);
    }
    else
    {
      #----------------------------------------
      # NON NodeManager based configuration
      #----------------------------------------

      ( $sCommand, $sCommandArgs) = $sServerStartProgram    # Separate
                                  =~ /(\S+)\s*(.*)/; # Cmd and args..

      if ( "Managed" eq $sServerRole )
      {
        $sCommandArgs ||= "$sServerName $sAdminURL"; # Assign arguments if not set earlier
      }

      $iReturnCode =  (Sys::RunWithEnvCmdInBG( "",                   #EnvFile
                                                       0,                    #Source User env
                                                       $sCommand,            #Program
                                                       $sCommandArgs,     #Arguments
                                                       \$iExitCode,
                                                       $sUser,
                                                       $sWLSOutFile,
                                                       \$bTimedOut,
                                                       $iTimeOut,
                                                       dirname $sCommand));
    }

    if ( $iReturnCode )
    {
      #------------------------
      # Command got executed.
      #------------------------
      if ( $iExitCode )
      {
        #------------------
        # Command failed.
        #------------------
        &logError(20044,"<%s>:[%s] command returned exit code [%s].",
                  $sMyName,$sCommand,$iExitCode);
        $iReturnCode = 0;
      }
      else
      {
        #-------------------------------
        # Command executed successfully.
        #-------------------------------
        &logTrace(20045,"<%s>:[%s] command executed successfully.",
                              $sMyName,$sCommand);
            $iReturnCode = 1;
      }
    }
    else
    {
      if ( $bTimedOut )
      {
        #-------------------
        # Command timed out.
        #-------------------
        &logError(20046,"<%s>:[%s] did not complete within [%s] seconds.",
                  $sMyName,$sCommand,$iTimeOut);
      }
      else
      {
        #-------------------
        # Command failed.
        #-------------------
        &logError(20047,"<%s>:[%s] command failed.",
                $sMyName,$sCommand);
      }
      $iReturnCode = 0;
    }
  

  if ( 1 == $iReturnCode )
  {
    #----------------------------------------------------------------------
    # Execution was successful.
    # If ServerRole is NodeManager we need to write its pid to our pidfile.
    # eg. /tmp/.VRTSWebLogic/WLS90Admin_nodemanager.pid
    #----------------------------------------------------------------------

    if ( "NodeManager" eq $sServerRole )
    {
      my $sPidFile        = "/tmp/.VRTS$sResourceType/$sResourceName.pid";
      my $riPID;

      #---------------------------------------------------------
      # Get the pid of the running java process for this Server.
      #---------------------------------------------------------
      if ( WebLogic::IsWLSProcessRunning( $sBEA_HOME ,
                                           $sWlstScript,
                                           $sEntryPointName,
                                           $sAdminURL,
                                           $sDomainName,
                                           $sDomainDir, 
                                           $sServerName,
                                           $sServerRole,
                                           $sHost,
                                           $sPort,
                                           $snmHost,
                                           $snmPort,
                                           $iState,
                                           $iIState,
                                           \$riPID ) )
      {
        #---------------------------------------
        # Write the node manager's pid to file.
        #---------------------------------------
        if (open (FH , ">$sPidFile"))
        {
          print FH $riPID;
          &logTrace(20048 ,"<%s>:Wrote [%s] to PidFile [%s].", 
                                $sMyName , $riPID , $sPidFile);
        }
        else
        {
          &logWarn(20049 ,"<%s>:Failed to create pid file [%s]",$sMyName,$sPidFile);
        }
      }
    }

    #------------------------------------------------------------------------
    # Now let us allow the server to fully initialize so that it could start
    # responding to SLM as well as External Monitor if specified.
    #------------------------------------------------------------------------

    if ( DelayEntryPoint ( $iExpirationTime,
                           $iState,
                           $iIState,
                           $sUser,
                           $sWLSUser,
                           $sWLSPassword,                         
                           $snmUser,
                           $snmPassword,                         
                           $sBEA_HOME,
                           $sWlstScript,
                           $sHost,
                           $sPort,
                           $snmHost,
                           $snmPort,
                           $sServerName,
                           $sServerRole,
                           $sDomainName,
                           $sDomainDir, 
                           $sAdminURL,
                           $iSecondLevelMonitor,
                           $snmType,
                           $sMonitorProgram ) )
    {     
       #---------------------------------------------
       # Server is fully online within OnlineTimeout
       # Might fail first monitor
       #---------------------------------------------
       &logTrace(20050,"<%s>:Server completely online.",$sMyName);
    }     
    else
    {
       #----------------------------------------------------
       # Server is not fully online within OnlineTimeout
       # Might fail first monitor
       #----------------------------------------------------
       &logTrace(20051,"<%s>:Server not completely online.",$sMyName);
       $iReturnCode = 0;
    }
  }
  return $iReturnCode;
}

=pod

 -------------------------------------------------------------------------
 Function : bool Stop(IN Integer State,
                      IN Integer IState,
                      IN String  BEA_HOME,
                      IN String  WlstScript,
                      IN String  User,
                      IN String  WLSUser,
                      IN String  WLSPassword,
                      IN String  nmUser,
                      IN String  nmPassword,                      
                      IN String  ListenAddressPort,
                      IN String  nmListenAddressPort,
                      IN String  ServerName,
                      IN String  ServerRole,
                      IN String  AdminURL,
                      IN String  DomainName,
                      IN String  DomainDir,
                      In String  EntryPointName,
                      In Integer ExpirationTime )

 Description : Stop logic needed for offline entry pt

 Parameters :

  $sState                  State variable 
  $sIState                 IState variable
  $sBEA_HOME               BEA Home Directory
  $sWlstScript             WLST Script Location
  $sUser                   Unix User
  $sWLSUser                WebLogic Server User
  $sWLSPassword            WebLogic Server Password
  $snmUser                Nodemanager User (shared secret)
  $snmPassword            Nodemanager Password  (shared secret)
  $sListenAddressPort      ListenAddress and Port of this WLS Server
  $snmListenAddressPort    ListenAddress and Port of the Node Manager
  $sServerName             WLS Server Name
  $sServerRole             WLS Server Role
  $sAdminURL               Administrative Server URL
  $sDomainName             WebLogic Domain Name
  $sDomainDir              WebLogic Domain Directory 
  $sEntryPointName         Entry Point Name
  $iExpirationTime         Expiration Time

 Returns : boolean value in the set of {0,1} where
           0 false
           1 true

 -------------------------------------------------------------------------

=cut

sub Stop {
  #----------------
  # Get Arguments..
  #----------------
  my ( $iState,
       $iIState,
       $sBEA_HOME,
       $sWlstScript,
       $sServerStopProgram,
       $sUser,
       $sWLSUser,
       $sWLSPassword,
       $snmUser,
       $snmPassword,       
       $sListenAddressPort,
       $snmListenAddressPort,
       $sServerName,
       $sServerRole,
       $sAdminURL,
       $nShutdownTimeout,
       $sDomainName,
       $sDomainDir,   
       $sEntryPointName,
       $iExpirationTime  ) = @_;
  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20052, "Subroutine <%s> called with:
  State                    [%s]
  iState                   [%s]
  BEA_HOME                 [%s]
  WlstScript               [%s] 
  ServerStopProgram        [%s] 
  User                     [%s]
  WLSUser                  [%s]
  WLSPassword              [%s]
  nmUser                   [%s]
  nmPassword               [%s]  
  ListenAddressPort        [%s]
  nmListenAddressPort      [%s]
  ServerName               [%s]
  ServerRole               [%s]
  AdminURL                 [%s]
  ShutdownTimeout          [%s]
  DomainName               [%s]
  DomainDir                [%s]
  EntryPointName           [%s]
  ExpirationTime           [%s]",
  $sMyName, $iState, $iIState, $sBEA_HOME, $sWlstScript, $sServerStopProgram, $sUser, 
  $sWLSUser, $sWLSPassword, $snmUser, $snmPassword, $sListenAddressPort, $snmListenAddressPort, $sServerName, 
  $sServerRole, $sAdminURL, $nShutdownTimeout, $sDomainName, $sDomainDir, $sEntryPointName, $iExpirationTime);

  #--------------------------------------
  # Set Return Code Optimistically to 1..
  #--------------------------------------
  my $iReturnCode = 1; # Success..

  my ($sHost,$sPort)     = split(/:/,$sListenAddressPort);
  my ($snmHost,$snmPort) = split(/:/,$snmListenAddressPort);
  my $riPID;

  if ( WebLogic::IsWLSProcessRunning( $sBEA_HOME ,
                                       $sWlstScript,
                                       $sEntryPointName,
                                       $sAdminURL,
                                       $sDomainName,
                                       $sDomainDir,
                                       $sServerName,
                                       $sServerRole,
                                       $sHost,
                                       $sPort,
                                       $snmHost,
                                       $snmPort,
                                       $iState,
                                       $iIState,
                                       \$riPID ) )
  {
    #------------------------------------------------------
    # WebLogic Server Process running .
    #------------------------------------------------------
    if ( "NodeManager" eq $sServerRole )
    {
      #------------------------------------------------------
      # NodeManager processes need to be killed for stopping.
      #------------------------------------------------------

      &logInfo( 20053, 
                             "<%s>:Killing Node Manager Server process [%s]",
                             $sMyName , $riPID);
      kill 9, $riPID;
    }
    else
    {
      #----------------------------------------
      # Admin or Managed Server. Need to use 
      # connect(), shutdown() api via wlst.sh
      #----------------------------------------

      my $sResourceType   = $ENV{VCS_LOG_AGENT_NAME};
      my $sResourceName   = $ENV{'VCS_LOG_RESOURCE_NAME'};

      my $sCommand       = $sWlstScript;
      my $sWLSOutFile    = "/tmp/.VRTS$sResourceType/$sResourceName"."_$sEntryPointName.out";
      my $sEnvFile       = "";
      my $sStopString    = "";
      my @sCommandArgsList;
      my $sCommandArgs;
      my $sCommandArgsForDisplay;
      my $bTimedOut;
      my $iTimeOut;
      my $iExitCode;
      my @lsCommandOut;
      my $sProtocol = "t3";

      my $sDecryptedPassword = "";
      
      if ( $sAdminURL )
      {
        ($sProtocol) = $sAdminURL =~ /(\w+):\/.*/ ;
        &logTrace(20054,"<%s>:Protocol extracted from AdminURL is [%s].",$sMyName, $sProtocol);
      }

      if ( "" ne $sWLSPassword )
      {
        $sDecryptedPassword = DecryptPassword($sWLSPassword);
        if ( ! $sDecryptedPassword )
        {
          &logError(20055,"<%s>:Could not decrypt the password",$sMyName);
          return 0;
        }
      }

      if ( $::gbNMBased )
      {
        $sStopString = "\"$sWLSUser\",\"$sDecryptedPassword\",\"${sProtocol}://$sListenAddressPort\"";
      }
      my $sStopStringForDisplay = "\"$sWLSUser\",XXXXXX,\"${sProtocol}://$sListenAddressPort\"";
  
      my ( $sName      ,
           $sPassword  ,
           $iUid       ,
           $iGid       ,
           $iQuota     ,
           $sComment   ,
           $sGcos      ,
           $sHome      ,
           $sUserShell ,
           $sExpire    ) = getpwnam($sUser);

      my $sShutdownString = "";
      if ( $nShutdownTimeout > 0 ){
        $sShutdownString = "'$sServerName', 'Server', 'false', $nShutdownTimeout";     
      }
      else {
        $sShutdownString = "";
      } 

      @sCommandArgsList = (
                           "connect\($sStopString\)\n",
                           "shutdown\($sShutdownString\)\n",
                           "exit\(\)\n"
                          );

      $sCommandArgsForDisplay = "<< _EOF_ 
connect($sStopStringForDisplay)
shutdown($sShutdownString)
exit()
_EOF_
";

      if ( $::gbNMBased )
      {
        &logTrace( 20056, "<%s>:Command Arguments [%s]",
                                    $sMyName , $sCommandArgsForDisplay);
      }
      else
      {
        ( $sCommand, $sCommandArgs) = $sServerStopProgram    # Separate
                                    =~ /(\S+)\s*(.*)/; # Cmd and args..
        if ( "Managed" eq $sServerRole )
        {
          ( my $sT3AdminURL = $sAdminURL ) =~ s/http:/t3:/;
          $sCommandArgs ||= "$sServerName $sT3AdminURL $sWLSUser $sDecryptedPassword"; # Assign arguments if not set earlier
        }
      }

      #------------------------
      # Remove the output file.
      #------------------------
      unlink ("$sWLSOutFile");

      $iTimeOut = $iExpirationTime - time;

      my $bDisplayOutput = 0;  #Dont display the password

      #------------
      # Set LANG=C 
      #------------
      my $sEnv_before = $ENV{LANG};
      $ENV{LANG} = "C" ;

      if ( $::gbNMBased )
      {
        $iReturnCode = RunCmdViaPipe ($sUser,
                                      $sCommand,
                                      $sWLSOutFile,
                                      \@sCommandArgsList,
                                      $sBEA_HOME,
                                      $sWlstScript, 
                                      \$iExitCode,
                                      $iTimeOut,
                                      \$bTimedOut);
        if ( $iReturnCode ) 
        {
          #------------------------
          # Command got executed.
          #------------------------
          if ( $iExitCode ) 
          {
            #------------------
            # Command failed.
            #------------------
            &logError(20057,"<%s>:[%s] command returned exit code [%s].",
                                           $sMyName,$sCommand,$iExitCode);
            $iReturnCode = 0;
           }
           else 
         {
            #-------------------------------
            # Command executed successfully.
            #-------------------------------
            &logTrace(20058,"<%s>:[%s] command executed successfully.",
                                                    $sMyName,$sCommand);
            $iReturnCode = 1;
           }
        }
        else 
        {
          if ( $bTimedOut ) 
          {
            #-------------------
            # Command timed out.
            #-------------------
            &logError(20059,"<%s>:[%s] did not complete within [%s] seconds.",
                                                 $sMyName,$sCommand,$iTimeOut);
          }
          else 
          {
            #-------------------
            # Command failed.
            #-------------------
            &logError(20060,"<%s>:[%s] command failed.",$sMyName,$sCommand);
          }
          $iReturnCode = 0;
         }
      }
      else
      {
         $iReturnCode = WebLogic::__RunWithEnvCmdWithOutputWithTimeOut(
                                                   $sCommand,
                                                   $sCommandArgs,
                                                   \$iExitCode,
                                                   $sUser,
                                                   \@lsCommandOut,
                                                   \$bTimedOut,
                                                   $iTimeOut,
                                                   $sWLSOutFile,
                                                   "",
                                                   $bDisplayOutput);
        if ( $iReturnCode )
        {
          #----------------------------------
          # Command executed.
          #----------------------------------
          if ( $iExitCode )
          {
            #-----------------
            # Command failed.
            #-----------------
            &logError (20061,"<%s>:[%s] failed with exit code [%s]",
                           $sMyName,$sServerStopProgram,$iExitCode);
          }
          else
          {
            #---------------------------------
            # Command executed successfully.
            #---------------------------------
            &logTrace (20062, "<%s>:Command [%s] executed successfully.",
                       $sMyName, $sServerStopProgram);
          }
        }
        else
        {
          #-----------------------------------
          # Problem during command execution.
          #-----------------------------------
          if ( $bTimedOut )
          {
            #--------------------
            # Command timed out.
            #--------------------
            &logError (20063,"<%s>:[%s] did not complete within [%s] seconds.",
                       $sMyName,$sServerStopProgram,$iTimeOut);
          }
          else
          {
            #--------------------
            # Command failed.
            #--------------------
            &logError (20064,"<%s>:[%s] failed.", $sMyName,$sServerStopProgram);
          }
        } 
      }

      #-----------------------------------
      # Re-set LANG to the original value
      #-----------------------------------
      defined $sEnv_before ? $ENV { LANG } = $sEnv_before:delete $ENV { LANG };

      #------------------------------------------------------------------
      # Implement DelayEntryPoint till OfflineTimeout value if the
      # Server appears to be running even after wlst.sh connect shutdown.
      #------------------------------------------------------------------
      my $iTmpSecondLevelMonitor   = 0;
      my $sTmpnmType               = "ssh"; # Value wont matter in this block.
      my $sTmpMonitorProgram       = "";
      if ($iReturnCode == 1 )
      {
        if ( DelayEntryPoint ( $iExpirationTime,
                               $iState,
                               $iIState,
                               $sUser,
                               $sWLSUser,
                               $sWLSPassword,
                               $snmUser,
                               $snmPassword,                         
                               $sBEA_HOME,
                               $sWlstScript,
                               $sHost,
                               $sPort,
                               $snmHost,
                               $snmPort,
                               $sServerName,
                               $sServerRole,
                               $sDomainName,
                               $sDomainDir, 
                               $sAdminURL,
                               $iTmpSecondLevelMonitor,
                               $sTmpnmType,
                               $sTmpMonitorProgram ) )
        {
          &logTrace( 20065, "<%s>:Successfully delayed the entry point.",$sMyName);
          $iReturnCode = 1;
        }
        else
        {
          &logError( 20066, "<%s>:Server process running even after DelayEntryPoint.",$sMyName);
          $iReturnCode = 0;
        }
      }

      if ( 1 != $iReturnCode ){ 
        &logInfo( 20067, "<%s>:Killing WebLogic Server process [%s]",
                          $sMyName , $riPID);
        kill 9, $riPID;
      }
    }
  }
  else
  {
      &logInfo( 20068, "<%s>:WebLogic Server process not found.",$sMyName);
  }

  #----------------------------------
  # Remove pid , slm counter files
  #----------------------------------
  my $sResourceType = $ENV{VCS_LOG_AGENT_NAME};
  my $sResourceName = $ENV{VCS_LOG_RESOURCE_NAME};
  my $sResDir       = "/tmp/.VRTS$sResourceType";
  &logTrace(20069,"<%s>:Unlinking [$sResDir/$sResourceName.slm] and [$sResDir/$sResourceName.pid]", $sMyName);
  unlink("$sResDir/$sResourceName.slm");
  unlink("$sResDir/$sResourceName.pid");

  return $iReturnCode;
}

=pod
-------------------------------------------------------------------------
 Function : bool NKill (IN Integer State,
                        IN Integer IState,
                        IN String  BEA_HOME,
                        IN String  WlstScript, 
                        IN String  User,
                        IN String  WLSUser,
                        IN String  WLSPassword,
                        IN String  nmUser,
                        IN String  nmPassword,                        
                        IN String  nmType,
                        IN String  ListenAddressPort,
                        IN String  nmListenAddressPort,
                        IN String  ServerName,
                        IN String  ServerRole,
                        IN String  AdminURL,
                        IN String  DomainName,
                        IN String  DomainDir,
                        IN String  EntryPointName,
                        IN Integer ExpirationTime  )

 Description : Kills the WebLogic server process by connecting to node manager and using nmKill() API

 Parameters :

  $iState                 State variable
  $iIState                IState variable
  $sBEA_HOME              BEA Home Directory
  $sWlstScript               Product Installation Directory
  $sUser                  Unix User
  $sWLSUser               WebLogic Server User
  $sWLSPassword           WebLogic Server Password
  $snmUser               Nodemanager User (shared secret)
  $snmPassword           Nodemanager Password  (shared secret)
  $snmType                Node Manager Type 
  $sListenAddressPort     ListenAddress and Port of this WLS Server
  $snmListenAddressPort   ListenAddress and Port of this Node manager
  $sServerName            WLS Server Name
  $sServerRole            WLS Server Role
  $sAdminURL              Administrative Server's URL
  $sDomainName            WebLogic Domain Name
  $sDomainDir             WebLogic Domain Directory  
  $sEntryPointName        Entry Point Name
  $iExpirationTime        Expiration Time

 
 Returns : boolean value in the set of {0,1} where
           0 false
           1 true
 -------------------------------------------------------------------------

=cut

sub NKill 
{

  #----------------
  # Get Arguments..
  #----------------

  my ( $iState,
       $iIState,
       $sBEA_HOME,
       $sWlstScript,
       $sUser,
       $sWLSUser,
       $sWLSPassword,
       $snmUser,
       $snmPassword,       
       $snmType,
       $sListenAddressPort,
       $snmListenAddressPort,
       $sServerName,
       $sServerRole,
       $sAdminURL,
       $sDomainName,
       $sDomainDir, 
       $sEntryPointName,
       $iExpirationTime  ) = @_;
  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20070, "Subroutine <%s> called with:
  State                    [%s]
  iState                   [%s]
  BEA_HOME                 [%s]
  WlstScript                  [%s] 
  User                     [%s]
  WLSUser                  [%s]
  WLSPassword              [%s]
  nmUser                  [%s]
  nmPassword              [%s]  
  nmType                   [%s]
  ListenAddressPort        [%s]
  nmListenAddressPort      [%s]
  ServerName               [%s]
  ServerRole               [%s]
  AdminURL                 [%s]
  DomainName               [%s]
  DomainDir                [%s]
  EntryPointName           [%s]
  ExpirationTime           [%s]",
  $sMyName, $iState, $iIState, $sBEA_HOME, $sWlstScript, $sUser, $sWLSUser, $sWLSPassword, $snmUser, $snmPassword,
  $snmType,$sListenAddressPort, $snmListenAddressPort, $sServerName, $sServerRole,
  $sAdminURL, $sDomainName, $sDomainDir, $sEntryPointName, $iExpirationTime);

  #--------------------------------------
  # Set Return Code Optimistically to 1..
  #--------------------------------------
  my $iReturnCode = 1; # Success..

  my ($sHost,$sPort)     = split(/:/,$sListenAddressPort);
  my ($snmHost,$snmPort) = split(/:/,$snmListenAddressPort);
  


  #----------------------------------------
  # Admin or Managed Server. Need to use
  # nmconnect(), nmkill() api via wlst.sh
  #----------------------------------------
   my $sResourceType   = $ENV{VCS_LOG_AGENT_NAME};
   my $sResourceName   = $ENV{'VCS_LOG_RESOURCE_NAME'};

   my $sCommand       = $sWlstScript;
   my $sWLSOutFile    = "/tmp/.VRTS$sResourceType/$sResourceName"."_$sEntryPointName.nkill.out";
   my $sEnvFile       = "";
   my @sCommandArgsList;
   my $sCommandArgsForDisplay;
   my $bTimedOut;
   my $iTimeOut;
   my $iExitCode;

   my $sDecryptedPassword = DecryptPassword($snmPassword);

   if ( ! $sDecryptedPassword )
   {
      &logError(20071,"<%s>:Could not decrypt the password",$sMyName);
      return 0;
   }


      @sCommandArgsList = (
                         "nmConnect\(\"$snmUser\",\"$sDecryptedPassword\",\"$snmHost\",\"$snmPort\",\"$sDomainName\",\"$sDomainDir\",\"$snmType\"\)\n",
                         "nmKill\(\"$sServerName\"\)\n",
                         "exit\(\)\n"
                       );



  $sCommandArgsForDisplay = "<< _EOF_"."nmConnect\(\"$snmUser\",XXXXXX,\"$snmHost\",\"$snmPort\",\"$sDomainName\",\"$sDomainDir\",\"$snmType\"\)\n"
                             ."nmKill\(\"$sServerName\"\)\n"."exit\(\)\n_EOF_" ;


      &logTrace( 20072, "<%s>:Command Arguments [%s]",
                                  $sMyName , $sCommandArgsForDisplay);
      #------------------------
      # Remove the output file.
      #------------------------
      unlink ("$sWLSOutFile");

      $iTimeOut = $iExpirationTime - time;

      $iReturnCode = RunCmdViaPipe ($sUser,
                                    $sCommand,
                                    $sWLSOutFile,
                                    \@sCommandArgsList,
                                    $sBEA_HOME,
                                    $sWlstScript,
                                    \$iExitCode,
                                    $iTimeOut,
                                    \$bTimedOut);
      if ( $iReturnCode )
      {
        #------------------------
        # Command got executed.
        #------------------------
        if ( $iExitCode )
        {
          #------------------
          # Command failed.
          #------------------
          &logError(20073,"<%s>:[%s] command returned exit code [%s].",
                           $sMyName,$sCommand,$iExitCode);
          $iReturnCode = 0;
        }
        else
        {
          #-------------------------------
          # Command executed successfully.
          #-------------------------------
          &logTrace(20074,"<%s>:[%s] command executed successfully.",
                          $sMyName,$sCommand);
          $iReturnCode = 1;
        }
      }
      else
      {
        if ( $bTimedOut )
        {
          #-------------------
          # Command timed out.
          #-------------------
          &logError(20075,"<%s>:[%s] did not complete within [%s] seconds.",
                           $sMyName,$sCommand,$iTimeOut);
        }
        else
        {
          #-------------------
          # Command failed.
          #-------------------
          &logError(20076,"<%s>:[%s] command failed.",$sMyName,$sCommand);
        }
        $iReturnCode = 0;
      }

      return $iReturnCode;
}



=pod

 -------------------------------------------------------------------------
 Function : bool Kill ( IN Integer CleanReason,
                        IN Integer State,
                        IN Integer IState,
                        IN String  BEA_HOME,
                        IN String  WlstScript,
                        IN String  User,
                        IN String  WLSUser,
                        IN String  WLSPassword,
                        IN String  nmUser,
                        IN String  nmPassword,                        
                        IN String  snmType,   
                        IN String  ListenAddressPort,
                        IN String  nmListenAddressPort,
                        IN String  ServerName,
                        IN String  ServerRole,
                        IN String  AdminURL,
                        IN String  DomainName,
                        IN String  DomainDir, 
                        IN String  EntryPointName,
                        IN Integer ExpirationTime  )

 Description : Forces shutdown of resource needed for clean entry point

 Parameters :

  $iCleanReason           Clean reason set by VCS
  $iState                 State variable
  $iIState                IState variable
  $sBEA_HOME              BEA Home Directory
  $sWlstScript               Product Installation Directory
  $sUser                  Unix User
  $sWLSUser               WebLogic Server User
  $sWLSPassword           WebLogic Server Password
  $snmUser               Nodemanager user (shared secret)
  $snmPassword           Nodemanager password (shared secret) 
  $snmType                Node Manager Type  
  $sListenAddressPort     ListenAddress and Port of this WLS Server
  $snmListenAddressPort   ListenAddress and Port of this Node manager
  $sServerName            WLS Server Name
  $sServerRole            WLS Server Role
  $sAdminURL              Administrative Server's URL
  $sDomainName            WebLogic Domain Name
  $sDomainDir             WebLogic Domain Directory
  $sEntryPointName        Entry Point Name
  $iExpirationTime        Expiration Time

 
 Returns : boolean value in the set of {0,1} where
           0 false
           1 true

 -------------------------------------------------------------------------

=cut

sub Kill 
{

  #----------------
  # Get Arguments..
  #----------------

  my ( $iCleanReason,
       $iState,
       $iIState,
       $sBEA_HOME,
       $sWlstScript, 
       $sServerStopProgram,
       $sUser,
       $sWLSUser,
       $sWLSPassword,
       $snmUser,
       $snmPassword,
       $snmType, 
       $sListenAddressPort,
       $snmListenAddressPort,
       $sServerName,
       $sServerRole,
       $sAdminURL,
       $sShutdownTimeout,
       $sDomainName,
       $sDomainDir,
       $sEntryPointName,
       $iExpirationTime  ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20077, "Subroutine <%s> called with:
  CleanReason              [%s]
  State                    [%s]
  IState                   [%s]
  BEA_HOME                 [%s]
  WlstScript                [%s]
  User                     [%s]
  WLSUser                  [%s]
  WLSPassword              [%s]
  nmUser                  [%s]
  nmPassword              [%s]
  nmType                   [%s]  
  ListenAddressPort        [%s]
  nmListenAddressPort      [%s]
  ServerName               [%s]
  ServerRole               [%s]
  AdminURL                 [%s]
  DomainName               [%s]
  DomainDir                [%s] 
  EntryPointName           [%s]
  ExpirationTime           [%s]",

  $sMyName, $iCleanReason, $iState, $iIState, $sBEA_HOME, $sWlstScript, $sUser,
  $sWLSUser, $sWLSPassword, $snmUser, $snmPassword, $snmType,$sListenAddressPort,
  $snmListenAddressPort, $sServerName, $sServerRole, $sAdminURL, $sDomainName, $sDomainDir,
  $sEntryPointName, $iExpirationTime  );

  #--------------------------------------
  # Set Return Code Optimistically to 1..
  #--------------------------------------
  my $iReturnCode        = 1; # Success..
  my ($sHost,$sPort)     = split(/:/,$sListenAddressPort);
  my ($snmHost,$snmPort) = split(/:/,$snmListenAddressPort);
  my $riPID;

  my $sResourceType   = $ENV{VCS_LOG_AGENT_NAME};
  my $sResourceName   = $ENV{VCS_LOG_RESOURCE_NAME};

  #----------------------------------------------
  # Create CleanHasJustRunIndicatorFile 
  # /tmp/AgentName.ResourceName.CleanHasJustRun
  #----------------------------------------------
  my $sCleanHasJustRunIndicatorFile = "/tmp/$sResourceType.$sResourceName.CleanHasJustrun";
  if ( open (TMP, ">> $sCleanHasJustRunIndicatorFile "))
  {
    close TMP;
  }
  else
  {
    &logError( 20078,  "<%s>:Couldn't create file [%s], OS error was [%s]",
                       $sMyName, $sCleanHasJustRunIndicatorFile, $! );
  }

  #----------------------------------------------------
  # Check whether WebLogic server process is running.
  #----------------------------------------------------
  
  if ( WebLogic::IsWLSProcessRunning( $sBEA_HOME ,
                                       $sWlstScript,
                                       $sEntryPointName,
                                       $sAdminURL,
                                       $sDomainName,
                                       $sDomainDir,
                                       $sServerName,
                                       $sServerRole,
                                       $sHost,
                                       $sPort,
                                       $snmHost,
                                       $snmPort,
                                       $iState,
                                       $iIState,
                                       \$riPID ) )
  {
    if ( "NodeManager" eq $sServerRole ) 
    {
      #---------------------------
      # ServerRole is NodeManager
      #---------------------------
      &logTrace( 20079, "<%s>:Unlinking pid file /tmp/.VRTS$sResourceType/$sResourceName.pid",$sMyName);
      unlink("/tmp/.VRTS$sResourceType/$sResourceName.pid");

      &logInfo( 20080, "<%s>:Killing WebLogic Node Manager process [%s].", $sMyName, $riPID);
      kill 9, $riPID;
    } #serverrole check
    else
    {
      #--------------------------------------------
      # ServerRole is Administrative or Managed 
      # NodeManager based configuration only
      #--------------------------------------------
      if ( $::gbNMBased && (  VCSDef_CleanReason::OFFLINE_HUNG == $iCleanReason ||
            VCSDef_CleanReason::OFFLINE_INEFFECTIVE == $iCleanReason)) 
      {
        
         #-----------------------------------------------------------------------
         # Connect to node manager and kill server using nmKill to eliminate 
         # possibility of restart by the node manager
         #-----------------------------------------------------------------------
         WebLogic::NKill  ($iState,
                            $iIState,
                            $sBEA_HOME,
                            $sWlstScript, 
                            $sUser,
                            $sWLSUser,
                            $sWLSPassword,
                            $snmUser,
                            $snmPassword,
                            $snmType,
                            $sListenAddressPort,
                            $snmListenAddressPort,
                            $sServerName,
                            $sServerRole,
                            $sAdminURL,
                            $sDomainName,
                            $sDomainDir,
                            $sEntryPointName,
                            $iExpirationTime) ; 
         if ( WebLogic::IsWLSProcessRunning
                                     ( $sBEA_HOME ,
                                       $sWlstScript,
                                       $sEntryPointName,
                                       $sAdminURL,
                                       $sDomainName,
                                       $sDomainDir,
                                       $sServerName,
                                       $sServerRole,
                                       $sHost,
                                       $sPort,
                                       $snmHost,
                                       $snmPort,
                                       $iState,
                                       $iIState,
                                       \$riPID ) )
         {
             &logError(20081,
                       "<%s>:WebLogic Server process still running after nmKill() attempt, hence killing WebLogic Server process [%s] through
 OS call(sending SIGTERM).",
                        $sMyName , $riPID);
             kill 15, $riPID;
             my $iSleepCounter = 1;
             my $bStillRunning = 1;
             while ($iSleepCounter < 6 && ( $bStillRunning == 1))
             {
               &logInfo(20082,"<%s>:Sleeping for 5 seconds.", $sMyName);
               sleep 5;
               if (WebLogic::IsWLSProcessRunning
                                       ( $sBEA_HOME ,
                                         $sWlstScript,
                                         $sEntryPointName,
                                         $sAdminURL,
                                         $sDomainName,
                                         $sDomainDir,
                                         $sServerName,
                                         $sServerRole,
                                         $sHost,
                                         $sPort,
                                         $snmHost,
                                         $snmPort,
                                         $iState,
                                         $iIState,
                                         \$riPID ) )
             {
               &logInfo(20083,"<%s>:Server PID running [%s].",$sMyName ,$riPID);
               $bStillRunning = 1;
             }
             else
             {
               $bStillRunning = 0;
             }
             $iSleepCounter++;
           }
           if ( $bStillRunning == 1 )
           {
                 &logError(20084,
                           "<%s>:WebLogic Server process still running after sending SIGTERM, hence killing WebLogic Server process [%s] thro
ugh OS call(sending SIGKILL).",
                            $sMyName , $riPID);
             kill 9, $riPID;
           }
         }
         else
         {
            &logInfo(20085,"<%s>:WebLogic Server process killed successfully through nmKill().",
                        $sMyName , $riPID);
         }
      }  
      else
      {
        #-----------------------------------------------------------------------
        # Initiate graceful shutdown before killing the processes.
        # Time for entry point is divided as follows:
        # 15 seconds for OS kill and clean up
        # Half remaining time for graceful shutdown
        # Half remaining time for nmKill()   
        #-----------------------------------------------------------------------
        my $iCurrTime = time;
        my $iExpirationTimeStop = $iCurrTime + ($iExpirationTime - $iCurrTime)/2;

        &logTrace( 20086, "<%s>:Current Time is [%s], Expiration time for Graceful Stop and NKill combined is [%s], Expiration time calculated for Graceful Stop is [%s]. Now attempting Graceful Stop of WebLogic Server.",$sMyName,$iCurrTime,$iExpirationTime,$iExpirationTimeStop);

        if ( WebLogic::Stop( $iState,
                            $iIState,
                            $sBEA_HOME,
                            $sWlstScript,
                            $sServerStopProgram,
                            $sUser,
                            $sWLSUser,
                            $sWLSPassword,
                            $snmUser,
                            $snmPassword,
                            $sListenAddressPort,
                            $snmListenAddressPort,
                            $sServerName,
                            $sServerRole,
                            $sAdminURL,
                            $sShutdownTimeout,
                            $sDomainName,
                            $sDomainDir,
                            $sEntryPointName,
                            $iExpirationTimeStop) ) 
        {
          &logInfo( 20087, "<%s>:Graceful shutdown of this WebLogic Server succeeded.",$sMyName);
        }
        else
         {
          &logError( 20088, "<%s>:Graceful shutdown of this WebLogic Server failed.",$sMyName);
          if ( $::gbNMBased )
          {
            &logError( 20089, "<%s>:Graceful shutdown of this WebLogic Server failed, trying through nmKill() API next.",$sMyName);
            #-----------------------------------------------------------------------
            # Connect to node manager and kill server using nmKill to eliminate 
            # possibility of restart by the node manager
            #-----------------------------------------------------------------------
            WebLogic::NKill  ( $iState,
                                $iIState,
                                $sBEA_HOME,
                                $sWlstScript,
                                $sUser,
                                $sWLSUser,
                                $sWLSPassword,
                                $snmUser,
                                $snmPassword,                                
                                $snmType, 
                                $sListenAddressPort,
                                $snmListenAddressPort,
                                $sServerName,
                                $sServerRole,
                                $sAdminURL,
                                $sDomainName,
                                $sDomainDir,
                                $sEntryPointName,
                                $iExpirationTime) ; 
          }
          if ( WebLogic::IsWLSProcessRunning
                                     ( $sBEA_HOME ,
                                       $sWlstScript,
                                       $sEntryPointName,
                                       $sAdminURL,
                                       $sDomainName,
                                       $sDomainDir,  
                                       $sServerName,
                                       $sServerRole,
                                       $sHost,
                                       $sPort,
                                       $snmHost,
                                       $snmPort,
                                       $iState,
                                       $iIState,
                                       \$riPID ) )
          {
             &logError(20090,
                       "<%s>:WebLogic Server process still running, killing WebLogic Server process [%s] through OS call.",
                       $sMyName , $riPID);
             kill 9, $riPID;
          }
          else
          {
             &logTrace(20091,"<%s>:WebLogic Server process not running.",
                        $sMyName);
          }
        }
      }
    }
  }
  else
  {
    #--------------------------------
    # WLS server process not running.
    #--------------------------------
    &logInfo( 20092, "<%s>:WebLogic Server process not found.", $sMyName);
  }

  #----------------------------------
  # Remove slm counter files
  #----------------------------------
  my $sResDir = "/tmp/.VRTS$sResourceType";
  &logTrace(20093,"<%s>:Unlinking [$sResDir/$sResourceName.slm] [$sResDir/$sResourceName.pid]", $sMyName);
  unlink("$sResDir/$sResourceName.slm");
  unlink("$sResDir/$sResourceName.pid");

  return $iReturnCode;
}

=pod

 -------------------------------------------------------------------------
 Function : int GetResourceState ( IN Integer State,
                                   IN Integer IState , 
                                   IN String  EntryPointName, 
                                   IN String  AdminURL ,
                                   IN String  BEA_HOME , 
                                   IN String  WlstScript,
                                   IN String  ListenAddressPort , 
                                   IN String  nmListenAddressPort , 
                                   IN String  ServerName , 
                                   IN String  ServerRole , 
                                   IN String  DomainName ,
                                   IN String  DomainDir , 
                                   IN String  nmType , 
                                   IN String  WLSUser , 
                                   IN String  WLSPassword , 
                                   IN String  User , 
                                   IN Integer SecondLevelMonitor , 
                                   IN String  MonitorProgram , 
                                   IN Integer ExpirationTime )

 Description : 
  Determines the true resources  state  in accordance   to  all of  the 
  monitoring related attribute settings. It first performs socket level 
  checking of the  WLS  resource.  Only  if that  is successful does it 
  perform any further tests.  

  Next, if the SecondLevelMonitor attribute is set, it will perform the 
  defined  SecondLevelMonitor test.  If this fails, it will not perform 
  any further  tests and will return PARTIAL_ONLINE (as the first level 
  checks did pass).

  Next, if the MonitorProgram attribute is set, it  will execute it. If 
  this fails, it will  not  perform any further tests  and  will return 
  PARTIAL_ONLINE (as all of the previous level checks did pass).

 Parameters :

  $iState                 State variable
  $iIState                IState variable
  $sEntryPointName        Entry Point Name
  $sAdminURL              Administrative Server's URL
  $sBEA_HOME              BEA Home Directory
  $sWlstScript               Product Installation Directory
  $sListenAddressPort     ListenAddress and Port of this WLS Server
  $snmListenAddressPort   ListenAddress and Port of this Node manager
  $sServerRole            WLS Server Role
  $sServerName            WLS Server Name
  $sDomainName            WLS Domain Name 
  $sDomainDir             WLS Domain Dir   
  $snmType                Node Manager Type
  $sWLSUser               WebLogic Server User
  $sWLSPassword           WebLogic Server Password
  $sUser                  Unix User
  $iSecondLevelMonitor    Second Level Monitor
  $sMonitorProgram        External Monitor Program
  $iExpirationTime        Expiration Time

 Returns : Integer value as defined below:

 100  OFFLINE                                  VCSDef_MonitorExit::OFFLINE
 105  ONLINE with confidence level 50   VCSDef_MonitorExit::PARTIAL_ONLINE
 110  ONLINE with confidence level 100          VCSDef_MonitorExit::ONLINE
 99   UNKNOWN; State couldn't be determined    VCSDef_MonitorExit::UNKNOWN

 -------------------------------------------------------------------------

=cut

sub GetResourceState {
  #----------------
  # Get Arguments..
  #----------------
  my ( $iState,
       $iIState , 
       $sEntryPointName, 
       $sAdminURL ,
       $sBEA_HOME , 
       $sWlstScript,
       $sListenAddressPort , 
       $snmListenAddressPort , 
       $sServerName , 
       $sServerRole , 
       $sDomainName , 
       $sDomainDir , 
       $snmType , 
       $sWLSUser , 
       $sWLSPassword , 
       $snmUser , 
       $snmPassword , 
       $sUser , 
       $iSecondLevelMonitor ,
       $sMonitorProgramAndArgs , 
       $iExpirationTime, 
       $level_one,
       $level_two ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20094, "Subroutine <%s> called with arguments:
  State                    [%s]
  iState                   [%s]
  EntryPointName           [%s]
  AdminURL		   [%s]
  BEA_HOME                 [%s]
  WlstScript               [%s]  
  ListenAddressPort        [%s]
  nmListenAddressPort      [%s]
  ServerName               [%s]
  ServerRole               [%s]
  DomainName               [%s]
  DomainDir                [%s] 
  nmType                   [%s]
  WLSUser                  [%s]
  WLSPassword              [%s]
  nmUser                   [%s]
  nmPassword               [%s]
  User                     [%s]
  SecondLevelMonitor       [%s]
  MonitorProgramAndArgs    [%s]
  ExpirationTime           [%s]
  level_one                [%s]
  level_two                [%s]",
  $sMyName, $iState, $iIState , $sEntryPointName, $sAdminURL, $sBEA_HOME , $sWlstScript, $sListenAddressPort,
  $snmListenAddressPort , $sServerName , $sServerRole , $sDomainName, $sDomainDir,
  $snmType , $sWLSUser , $sWLSPassword , $snmUser, $snmPassword, 
  $sUser , $iSecondLevelMonitor, $sMonitorProgramAndArgs , $iExpirationTime, $level_one, $level_two );


  #-----------------------------
  # Execute first level check.
  #-----------------------------
  my $iReturnCode; 
  if ( 1 == $level_one ) {
    $iReturnCode = WebLogic::GetWLSState( $sBEA_HOME,
                                              $sWlstScript,
                                              $sEntryPointName,
                                              $sAdminURL,
                                              $sUser,
                                              $sWLSUser,
                                              $sWLSPassword,
                                              $sServerName,
                                              $sServerRole,
                                              $sDomainName,
                                              $sDomainDir, 
                                              $sListenAddressPort,
                                              $snmListenAddressPort,
                                              $iExpirationTime,
                                              $iSecondLevelMonitor,
                                              $sMonitorProgramAndArgs,
                                              $snmType,
                                              $iState,
                                              $iIState );
  
    if( $iReturnCode == VCSDef_MonitorExit::ONLINE )
    {
      if ( "NodeManager" ne $sServerRole )
      {
        &logTrace(20095, "<%s>:Found WLS processes running for Server [%s]",
                           $sMyName, $sServerName );
      }
      else
      {
        &logTrace(20096, "<%s>:Found WLS processes running for NodeManager",
                           $sMyName );
      }
    }
    elsif( $iReturnCode == VCSDef_MonitorExit::OFFLINE )
    {
      if ( "NodeManager" ne $sServerRole )
      {
        &logTrace(20097, "<%s>:Found no WLS processes for Server [%s]",
                          $sMyName, $sServerName );
      }
      else
      {
        &logTrace(20098, "<%s>:Found no WLS processes for NodeManager",
                          $sMyName );
      }
      return $iReturnCode ;
    }
    else
    {
      &logError(20099, "<%s>:Could not determine the resource's state.",$sMyName);
      return VCSDef_MonitorExit::UNKNOWN ;
    }
  
  }

  #---------------------------------------
  # Do Second level monitoring if enabled.
  #---------------------------------------
  if ( -300 == $level_two ) {
     $level_two = RunSecondLevelMonitor ( $iSecondLevelMonitor );  
     &logTrace ( 20307, "<%s> Traditional second level monitoring, SLM flag for this monitor set to [%s] ", $sMyName, $level_two  );
  }

  if ( $level_two >= 1 )
  {
    #-------------------------------------------------------------------
    # Determine how much time to give it to run;  the calcuation used
    # will only allow the site provided MonitorProgram to run if there
    # is at least two times "EPOP" left before the MonitorProgramTimeout
    # would be hit.  This is six seconds when EPOP is set to 3, the
    # hard-coded default.  (one unit of EPOP has already been subtracted
    # from the ExpirationTime earlier)
    #-------------------------------------------------------------------

    my $iTimeOut = $iExpirationTime - time ;

    if( $iTimeOut < EPOP )
    {
      &logInfo( 20100,"<%s>:Not enough time left to attempt to run the SecondLevelMonitor this monitor interval", $sMyName );
      return VCSDef_MonitorExit::ONLINE ;
    }

    my $sHost;
    my $sPort;
    if ( "NodeManager" eq $sServerRole)
    {
      ($sHost,$sPort) = split(/:/,$snmListenAddressPort);       
    }
    else
    {
      ($sHost,$sPort) = split(/:/,$sListenAddressPort);       
    }

    $iReturnCode = WebLogic::SecondLevelMonitor ($sUser,
                                                  $sWLSUser,
                                                  $sWLSPassword,
                                                  $snmUser,
                                                  $snmPassword,
                                                  $sHost,
                                                  $sPort,
                                                  $sServerRole,
                                                  $sDomainName,
                                                  $sDomainDir, 
                                                  $sBEA_HOME,
                                                  $sWlstScript,
                                                  $snmType,
                                                  $sAdminURL,
                                                  $iTimeOut);

    if( $iReturnCode == VCSDef_MonitorExit::OFFLINE )
    {
      #-------------------------------------------------------------------
      # WLS processes are there, but the SecondLevelMonitor failed
      # Return PARTIAL_ONLINE
      #-------------------------------------------------------------------
      return VCSDef_MonitorExit::PARTIAL_ONLINE ;
    }
    elsif( $iReturnCode == VCSDef_MonitorExit::UNKNOWN   )
    {
      return VCSDef_MonitorExit::UNKNOWN ;
    }
    elsif( $iReturnCode == VCSDef_MonitorExit::ONLINE )
    {
      #--------------------------------------------------------------
      # Fall through to check if there is a MonitorProgram defined.
      #--------------------------------------------------------------
    }
    else
    {
      &logError( 20101, "<%s>:Second level monitor procedures for the WebLogic resource failed to run or timedout", $sMyName );
      return VCSDef_MonitorExit::UNKNOWN;
    }
  }

  #--------------------------------------------------
  # Run site provided monitor program if specified.
  #--------------------------------------------------
  my ($sMonitorProgram,$sMonitorProgramArgs) = 
                           split(/\s/,$sMonitorProgramAndArgs,2);
  if( $sMonitorProgramAndArgs ne "" )
  {
    if ( ! -e $sMonitorProgram )
    {
      #------------------------------------------------
      # MonitorProgram not found or is not executable.
      #------------------------------------------------

      &logError(20102, "<%s>: MonitorProgram [%s] not executable. Check file existence and permissions.", $sMyName, $sMonitorProgram);

      return VCSDef_MonitorExit::UNKNOWN;
    }

    if(! defined($sMonitorProgramArgs) ) {
      $sMonitorProgramArgs = "";
    }

    my $iMonitorProgramExitCode = 0;
    my @lsCommandOut            = ();
    my $bTimedOut               = 0; # setting timeout flag to true for now.

    #-------------------------------------------------------------------
    # Determine how much time to give it to run;  the calcuation used
    # will only allow the site provided MonitorProgram to run if there
    # is at least two times "EPOP" left before the MonitorProgramTimeout
    # would be hit.  This is six seconds when EPOP is set to 3, the
    # hard-coded default.  (one unit of EPOP has already been subtracted
    # from the ExpirationTime earlier)
    #-------------------------------------------------------------------
    my $iMonitorProgramTimeout = $iExpirationTime - time - EPOP - EPOP ;
    if( $iMonitorProgramTimeout < EPOP )
    {
      &logInfo( 20103,"<%s>:Not enough time left to attempt to run the defined MonitorProgram this monitor interval", $sMyName );
      return VCSDef_MonitorExit::ONLINE;
    }

    &logTrace(20104,"<%s>:About to run [%s] with arguments [%s]",
                                $sMyName,$sMonitorProgram,$sMonitorProgramArgs);

     if (Sys::RunWithEnvCmdWithOutputWithTimeOut( "",
                                                  0,
                                                  $sMonitorProgram,
                                                  $sMonitorProgramArgs,
                                                  \$iMonitorProgramExitCode,
                                                  $sUser,
                                                  \@lsCommandOut,
                                                  \$bTimedOut,
                                                  $iMonitorProgramTimeout,
                                                  "",
                                                  $sBEA_HOME))
    {
      #--------------------------------------------------------------------------
      # The MonitorProgram was successfully run; check the exit code it returned.
      #--------------------------------------------------------------------------
      if( ( VCSDef_MonitorExit::ONLINE == $iMonitorProgramExitCode ) || 
          ( $iMonitorProgramExitCode == 0 ) )
      {

        $iReturnCode = VCSDef_MonitorExit::ONLINE;
        &logTrace( 20105,"<%s>:MonitorProgram [%s] with args [%s] and user context [%s] indicated ONLINE (exit code [%s])",
                            $sMyName,
                            $sMonitorProgram,
                            $sMonitorProgramArgs,
                            $sUser,
                            $iMonitorProgramExitCode );

      }
      elsif( ( $iMonitorProgramExitCode == VCSDef_MonitorExit::OFFLINE ) || 
             ( $iMonitorProgramExitCode == 1 ) )
      {
        #----------------------------------------------------------------------
        # We must return PARTIAL_ONLINE to reflect that the first level process
        # check (and possibly the SecondLevelMonitor check) did succeed.
        #----------------------------------------------------------------------

        $iReturnCode = VCSDef_MonitorExit::PARTIAL_ONLINE;
        &logError( 20106,"<%s>:MonitorProgram [%s] with args [%s] and user context [%s] indicated OFFLINE (exit code [%s])",
                            $sMyName,
                            $sMonitorProgram,
                            $sMonitorProgramArgs,
                            $sUser,
                            $iMonitorProgramExitCode);
        VCSagentFW::logOutputFromCommand( "$sMonitorProgram $sMonitorProgramArgs",
                                                  $iMonitorProgramExitCode,
                                                  \@lsCommandOut ,
                                                  LogInt::LLError);
        $iReturnCode = VCSDef_MonitorExit::PARTIAL_ONLINE;
      }
      else
      {
        $iReturnCode = VCSDef_MonitorExit::UNKNOWN;
        &logWarn( 20107, "<%s>:MonitorProgram [%s] with args [%s] and user context [%s] did not indicate ONLINE nor OFFLINE (it provided exit code [%s])",
                           $sMyName,
                           $sMonitorProgram,
                           $sMonitorProgramArgs,
                           $sUser,
                           $iMonitorProgramExitCode);
        VCSagentFW::logOutputFromCommand( "$sMonitorProgram $sMonitorProgramArgs",
                                                  $iMonitorProgramExitCode,
                                                  \@lsCommandOut,
                                                  LogInt::LLWarn);
      }
    }
    else
    {
      if( $bTimedOut )
      {
        #----------------------------------------------------------------------
        # Command timed out!  Return UNKNOWN as the first (and possibly second)
        # level monitoring did work -- it is considered safer to return UNKNOWN
        # as the service may be generally OK, and the MonitorProgram didn't
        # determine for sure that the service was not up, as the defined
        # methodology to do so (as implemented in the MonitorProgram) could not
        # be run successfully (it timed out).
        #----------------------------------------------------------------------
        &logError( 20108, "<%s>:MonitorProgram [%s] with args [%s] and user context [%s] failed to run (timed out)",
                            $sMyName,
                            $sMonitorProgram,
                            $sMonitorProgramArgs,
                            $sUser );
        &logTrace( 20109,"<%s>:Sleeping till had kills the entry point.",$sMyName);
        sleep (1000) while(1);
        $iReturnCode = VCSDef_MonitorExit::UNKNOWN;
      }
      else
      {
        #----------------------------------------------------------------------
        # The MonitorProgram was not successfully run; The call to the
        # MonitorProgram failed (but did NOT timeout).  Return UNKNOWN for the
        # same reasons as explained above.
        #----------------------------------------------------------------------
        &logError( 20110, "<%s>:MonitorProgram [%s] with args [%s] and user context [%s] failed to run (problems in call to 'Sys::RunWithEnvCmdWithOutputWithTimeOut')",
                            $sMyName,
                            $sMonitorProgram,
                            $sMonitorProgramArgs,
                            $sUser );
        $iReturnCode = VCSDef_MonitorExit::UNKNOWN;
      }
    }
  }
  &logTrace( 20111,"<%s>:Returning [%s]", $sMyName , $iReturnCode);
  return $iReturnCode ;
}
 

=pod

 -------------------------------------------------------------------------
 Function : int GetWLSState ( IN String  BEA_HOME,
                              IN String  WlstScript,
                              IN String  EntryPointName, 
                              IN String  AdminURL , 
                              IN String  User , 
                              IN String  WLSUser , 
                              IN String  WLSPassword , 
                              IN String  ServerName , 
                              IN String  ServerRole , 
                              IN String  DomainName ,
                              IN String  DomainDir ,   
                              IN String  ListenAddressPort , 
                              IN String  nmListenAddressPort , 
                              IN Integer ExpirationTime ,
                              IN Integer SecondLevelMonitor , 
                              IN String  MonitorProgram , 
                              IN String  nmType , 
                              IN Integer State ,
                              IN Integer IState )

 Description : First level monitor to determine state of resource.

 This subroutines first checks  whether the IP from (nm)ListenAddressPort 
 if plumbed on the machine. If its not up, it simply will return OFFLINE.

 Next  it does  a  socket  connection  to the (nm)ListenAddressPort to do
 first level check. If socket  connection is successful and ServerRole is
 Administrative  or  Managed  it  returns  ONLINE  immediately.   

 If  the ServerRole  is  NodeManager  , it will also do process  check by  
 using the pid file for NodeManager which  we created  in  online. If the  
 nodemanager pid file is not available or  is stale , it will also create 
 the pid file with correct pid for the running nodemanager process.

 If a Managed or Administrative server process  is found to be  restarted
 by its nodemanager the subroutine  also has  the logic to give a waiting
 time for  such restarted process to initialize without occurring a fault
 if during such initialization its FLM fails.

 Parameters :

  $sBEA_HOME              BEA Home Directory
  $sWlstScript               Product Installation Directory
  $sEntryPointName        Entry Point Name
  $sAdminURL              Administrative Server's URL
  $sUser                  Unix User
  $sWLSUser               WebLogic Server User
  $sWLSPassword           WebLogic Server Password
  $sServerName            WLS Server Name
  $sServerRole            WLS Server Role
  $sDomainName            WLS Domain Name
  $sDomainDir             WLS Domain Directory
  $sListenAddressPort     ListenAddress and Port of this WLS Server
  $snmListenAddressPort   ListenAddress and Port of this Node manager
  $iExpirationTime        Expiration Time
  $iSecondLevelMonitor    Second Level Monitor
  $snmType                Node Manager Type
  $sMonitorProgram        External Monitor Program
  $iState                 State variable
  $iIState                IState variable

 Returns : Integer value as defined below:

 99   UNKNOWN                          VCSDef_MonitorExit::UNKNOWN
 100  OFFLINE                          VCSDef_MonitorExit::OFFLINE
 110  ONLINE                           VCSDef_MonitorExit::ONLINE

 -------------------------------------------------------------------------

=cut

sub GetWLSState
{

  my ( $sBEA_HOME,
       $sWlstScript,  
       $sEntryPointName,
       $sAdminURL,
       $sUser,
       $sWLSUser,
       $sWLSPassword,
       $sServerName,
       $sServerRole,
       $sDomainName,
       $sDomainDir,  
       $sListenAddressPort,
       $snmListenAddressPort,
       $iExpirationTime,
       $iSecondLevelMonitor,
       $sMonitorProgram,
       $snmType,
       $iState,
       $iIState ) = @_ ;

  #-----------------
  # Announce Entry
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace( 20112, "Subroutine <%s> called with:
  BEA_HOME                 [%s] 
  WlstScript               [%s]
  EntryPointName           [%s]
  AdminURL                 [%s]
  User                     [%s]
  WLSUser                  [%s]
  WLSPassword              [%s]
  ServerName               [%s]
  ServerRole               [%s]
  DomainName               [%s]
  DomainDir                [%s]
  ListenAddressPort        [%s]
  nmListenAddressPort      [%s]
  ExpirationTime           [%s]
  MonitorProgram           [%s]
  State                    [%s]
  IState                   [%s]" ,
  $sMyName,$sBEA_HOME,$sWlstScript,$sEntryPointName,$sAdminURL,$sUser,$sWLSUser,$sWLSPassword,
  $sServerName,$sServerRole,$sDomainName,$sDomainDir,$sListenAddressPort,
  $snmListenAddressPort,$iExpirationTime,$sMonitorProgram,$iState,$iIState);

  my $sHostPort   = $sListenAddressPort;
  my $snmHostPort = $snmListenAddressPort;

  #----------------------------------------------------------------
  # Extracting IPs for checking whether it is plumbed on this node.
  #----------------------------------------------------------------
  my ( $sHost, $sIP, $sPort, $snmHost, $snmIP, $snmPort ) = ();

  if ( $snmHostPort =~ /\S/) {
    ($snmHost) = split(/:/,$snmHostPort);
    ($snmIP,$snmPort) = ExtractIP($snmHostPort) if $::gbNMBased;
  }

  if ( $sServerRole ne "NodeManager" )
  {
    if ( $sHostPort =~ /\S/) {
      ($sHost)      = split(/:/,$sHostPort);
      ($sIP,$sPort) = ExtractIP($sHostPort) if ($sHostPort);
    }
  }

  if ( ($sServerRole eq "NodeManager" ) && $snmIP && $snmPort ) {
    # All well
  }
  elsif ( ($sServerRole ne "NodeManager") && $sIP && $sPort) {
    # All well
  }
  else {
    #------------------------------------
    # Got a problem here. Return UNKNOWN
    #------------------------------------
    &logError( 20113, "<%s>:Should not reach here, check configuration.",$sMyName);
    return VCSDef_MonitorExit::UNKNOWN;
  }

  my $sCheckIP   = $sIP;
  my $sCheckPort = $sPort;
  if ( $sServerRole eq "NodeManager" )
  {
    $sCheckIP   = $snmIP;
    $sCheckPort = $snmPort;
  }

  if ( 0 == WebLogic::CheckIfIPExistsLocally($sCheckIP) )
  {
    #-------------------------------
    # Possibly on an offline node
    #-------------------------------
    &logTrace( 20114, "<%s>:IP [%s] Does not exist locally.",$sMyName, $sCheckIP );
    return VCSDef_MonitorExit::OFFLINE;
  }
  
  #-----------------------------------------------------------------------
  # IP for the resource is plumbed we are on a potential ONLINE node.
  # First try a socket connection.
  #-----------------------------------------------------------------------

  if ( 0 == Comms::CheckPort($sCheckIP,$sCheckPort) )
  {
    &logTrace( 20115, "<%s>:IP [%s] not listening on [%s].",
                                   $sMyName, $sCheckIP, $sCheckPort);

    #--------------------------------------------------------------------
    # If we are in online entry point, confirm WLS process is not running.
    #--------------------------------------------------------------------
    if ( "online" eq $sEntryPointName )
    {
        my $riPID;
        if ( WebLogic::IsWLSProcessRunning($sBEA_HOME ,
                                            $sWlstScript, 
                                            $sEntryPointName,
                                            $sAdminURL,
                                            $sDomainName,
                                            $sDomainDir,  
                                            $sServerName,
                                            $sServerRole,
                                            $sHost,
                                            $sPort,
                                            $snmHost,
                                            $snmPort,
                                            $iState,
                                            $iIState,
                                            \$riPID ) )
        {
          #-------------------------------------------------------------------
          # We are in online entry point. First level socket connection checks
          # failed but process is running. Return UNKNOWN.
          #-------------------------------------------------------------------
          &logWarn(20116,"<%s>:Process [%s] found to be running for Server.",$sMyName,$riPID);
          return VCSDef_MonitorExit::UNKNOWN;
        }
        else
        {
          return VCSDef_MonitorExit::OFFLINE;
        }
    }

    #--------------------------------------------------------------------
    # Socket connection failed, we need to return OFFLINE now , if the 
    # ServerRole is NodeManager else we have to see if the state of this 
    # resource was ONLINE previously in VCS.
    #--------------------------------------------------------------------

    if ( $sServerRole eq "NodeManager")
    {
      #---------------------------
      # ServerRole is NodeManager
      #---------------------------
      &logTrace( 20117, "<%s>:NodeManager IP [%s] not listening on [%s].",
                  $sMyName, $sCheckIP, $sCheckPort);

      return VCSDef_MonitorExit::OFFLINE;
    }
    else
    {
      #-----------------------------------------
      # ServerRole is Administrative or Managed
      #-----------------------------------------
      if ( $::gbNMBased && (VCSDef_ResourceState::ONLINE == $iState && (VCSDef_ResourceIState::WAITING_TO_GO_OFFLINE != $iIState)))
      {
         #-----------------------------------------------------------
         # The Server was ONLINE already in VCS. Probably the node
         #  manager has restarted this Server. Need to double check.
         #-----------------------------------------------------------
         if ( "offline" ne $sEntryPointName )
         {
           &logWarn( 20118, 
                     "<%s>:IP [%s] no longer listening on [%s].",
                     $sMyName, $sCheckIP, $sCheckPort);
         }

         my $riPID;
         if ( WebLogic::IsWLSProcessRunning($sBEA_HOME , 
                                             $sWlstScript,
                                             $sEntryPointName,  
                                             $sAdminURL,  
                                             $sDomainName, 
                                             $sDomainDir,
                                             $sServerName, 
                                             $sServerRole, 
                                             $sHost, 
                                             $sPort,
                                             $snmHost,
                                             $snmPort,
                                             $iState,
                                             $iIState,
                                             \$riPID ) )
         {
           #--------------------------
           # The process is running.
           #--------------------------
           &logWarn( 20119, "<%s>:[%s] was ONLINE but socket connection at [%s:%s] failed. Node manager might have restarted it. Allowing server PID [%s] to initialize.",
                            $sMyName,
                            $sServerName,
                            $sIP,
                            $sPort,
                            $riPID);

           #----------------------------------------------------
           # DelayEntryPoint Now we need to allow the Server to 
           # initialize till MonitorTimeout is reached
           #----------------------------------------------------

           my $iWLSProcessRunTime = GetWLSProcessRunTime($riPID);
           if ( ! $iWLSProcessRunTime )
            {
              &logTrace( 20120, "<%s>:Could not determine how long process [%s] is running.",$sMyName, $riPID );
             return VCSDef_MonitorExit::UNKNOWN;
            }

           if ( $::hWebLogicTypeValues{"MonitorTimeout"} < $iWLSProcessRunTime )
           {
             #-------------------------------------------------------------
             # Cannot wait as the MonitorTimeout value is already exceeded 
             # for this Process's runtime.
             #-------------------------------------------------------------
             return VCSDef_MonitorExit::OFFLINE;
           }

           my $iTimeRemaining = $iExpirationTime - time;
           if ( ($iTimeRemaining - 5) <= 0 )
            {
              &logTrace( 20121, "<%s>:Not enough time left [%s] to allow the process to initialize.",
                               $sMyName, $iTimeRemaining );
             return VCSDef_MonitorExit::UNKNOWN;
            }

            #-------------------------------------------------------
            # We have some time to allow the server to initialize.
            # Delay the Entry Point.
            #-------------------------------------------------------
            &logTrace( 20122, "<%s>:Delaying Entry Point.", $sMyName);
           if ( DelayEntryPoint ( $iExpirationTime,
                                  $iState,
                                  $iIState,
                                  $sUser,
                                  $sWLSUser,
                                  $sWLSPassword,
                                  $sBEA_HOME,
                                  $sWlstScript,
                                  $sHost,
                                  $sPort,
                                  $snmHost,
                                  $snmPort,
                                  $sServerName,
                                  $sServerRole,
                                  $sDomainName,
                                  $sDomainDir, 
                                  $sAdminURL,
                                  $iSecondLevelMonitor,
                                  $snmType,
                                  $sMonitorProgram ))
           {
              return VCSDef_MonitorExit::ONLINE;
            }
         }
      }
      return VCSDef_MonitorExit::OFFLINE;
    }
  }
  else
  {
    #-----------------------------------
    # socket connection is successful.
    #-----------------------------------

    if ( "NodeManager" eq $sServerRole )
    {
      #-----------------------------------------------------------------------
      # If the ServerRole is NodeManager and it got killed while its managed
      # application servers are running, its port will give a successful 
      # connection. Need to double check whether the node manager process
      # is really running.
      #-----------------------------------------------------------------------
      my $iPID;
      my $sResourceType   = $ENV{VCS_LOG_AGENT_NAME};
      my $sResourceName   = $ENV{VCS_LOG_RESOURCE_NAME};
      my $sPidFile        = "/tmp/.VRTS$sResourceType/$sResourceName.pid";
      my $iReturn         = 0;

      &logTrace(20123,"<%s>:Opening PidFile [%s]",
                                   $sMyName , $sPidFile);
      $iReturn = open (FH,"<$sPidFile");
      if ( $iReturn )
      {
         #----------------------------------------
         # CheckWLSProcessID(pid)
         # 1 Successful we are okay
         # 2 Stale pid file , recycle the pid file
         # 0 Not running exit OFFLINE.
         #----------------------------------------
         $iPID = <FH>;
         close FH;

        &logTrace(20124,"<%s>:Got pid [%s] from PidFile [%s]",
                                   $sMyName , $iPID , $sPidFile);
        $iReturn = CheckWLSProcessID( $sBEA_HOME,
                                      $sWlstScript, 
                                      $sEntryPointName,
                                      $sAdminURL,
                                      $sDomainName,
                                      $sDomainDir,
                                      $sServerName,
                                      $sServerRole,
                                      $sHost,
                                      $sPort,
                                      $snmHost,
                                      $snmPort,
                                      $iState,
                                      $iIState,
                                      $iPID );
         if ( 1 == $iReturn )
         {
           #---------------------------------------
           # pid file contains correct process id.
           #---------------------------------------
          &logTrace( 20125, "<%s>:Server process [%s] found running. PidFile [%s] reporting correct pid.",$sMyName, $iPID, $sPidFile);
         }
         elsif ( 2 == $iReturn )
         {
           #---------------------------------------
           # pid file contains zombie process id.
           #---------------------------------------
          &logTrace( 20126, "<%s>:Server process [%s] found to be zombie.",$sMyName, $iPID);
          return VCSDef_MonitorExit::OFFLINE;
         }
         elsif ( 3 == $iReturn )
         {
           #---------------------------------------
           # pid file is stale, server is running.
           #---------------------------------------
          &logTrace( 20127, "<%s>:Server process [%s] found running. PidFile was stale.",$sMyName, $iPID);
         }
         elsif ( 0 == $iReturn )
         {
           #---------------------------------------
           # server process is not running.
           #---------------------------------------
          return VCSDef_MonitorExit::OFFLINE;
         }
         close FH;
      }
      else
      {
        &logTrace( 20128, "<%s>:Cannot open pid file [%s] for reading.",$sMyName, $sPidFile);
         #--------------------------
         # Error opening pid file.
         #--------------------------
        if ( WebLogic::IsWLSProcessRunning($sBEA_HOME ,
                                            $sWlstScript, 
                                            $sEntryPointName,
                                            $sAdminURL,
                                            $sDomainName,
                                            $sDomainDir, 
                                            $sServerName,
                                            $sServerRole,
                                            $sHost,
                                            $sPort,
                                            $snmHost,
                                            $snmPort,
                                            $iState,
                                            $iIState,
                                            \$iPID ) )
        {
          &logTrace( 20129, "<%s>:Server IP [%s] is listening on [%s]",$sMyName, $sCheckIP ,$sCheckPort );
        }
         else
         {
          &logTrace( 20130, "<%s>:Server processes not running",$sMyName);
          return VCSDef_MonitorExit::OFFLINE;
         }
        if (! -d "/tmp/.VRTS$sResourceType" )
        {
          mkdir ("/tmp/.VRTS$sResourceType",0777);
          chmod (0777, "/tmp/.VRTS$sResourceType");
        }
        open ( FH , "> $sPidFile");
        print FH $iPID;
        &logTrace( 20131, "<%s>:Wrote [%s] to PidFile [%s]",
                                       $sMyName , $iPID , $sPidFile);
        close FH;
      }
      &logTrace( 20132, "<%s>:Server IP [%s] is listening on [%s]",$sMyName, $sCheckIP ,$sCheckPort );
      return VCSDef_MonitorExit::ONLINE;
    }
    return VCSDef_MonitorExit::ONLINE;
  }
}

=pod

 -------------------------------------------------------------------------

 Function : int IsWLSProcessRunning ( IN  String  BEA_HOME,
                                      IN  String  WlstScript,
                                      IN  String  EntryPointName,
                                      IN  String  AdminURL,
                                      IN  String  DomainName,
                                      IN  String  DomainDir,
                                      IN  String  ServerName,
                                      IN  String  ServerRole,
                                      IN  String  Host,
                                      IN  Integer Port,
                                      IN  String  nmHost,
                                      IN  Integer nmPort,
                                      IN  Integer State,
                                      IN  Integer IState,
                                      OUT Integer riPID )

 Description : Check whether the WLS processes are running on the system.
 This subroutine traces the process  table  of  the system  to  determine
 whether the WLS server is running on this node. It uses the following
 regex to filter out the WLS processes.
 
 AdminServer:  "^$sBEA_HOME.*java.*Name=$sServerName\\b.*$sDomainName.*Server\\b";
 Managed    :  "^$sBEA_HOME.*java.*Name=$sServerName\\b.*management.server=$sAdminURL.*$sDomainName";
 NodeManager:  "^$sBEA_HOME.*java.*ListenAddress=$snmHost\\b.*ListenPort=$snmPort\\b";

 Parameters :

  $sBEA_HOME              BEA Home Directory
  $sWlstScript               Product Installation Directory
  $sEntryPointName        Entry Point Name
  $sAdminURL              Administrative Server's URL
  $sDomainName            WLS Domain Name 
  $sDomainDir             WLS Domain Directory
  $sServerName            WLS Server Name
  $sServerRole            WLS Server Role
  $sHost                  Listen Address of WLS Server
  $sPort                  Listen Port of Node manager
  $snmHost                ListenAddress of Node Manager
  $snmPort                Port of Node manager
  $iState                 State variable
  $iIState                IState variable
  $riPID                  Process id

 Returns : Integer value as defined below:

 0 Failure         WLS Process not running for this resource.
 1 Success         WLS Process running for this resource.

 -------------------------------------------------------------------------

=cut
 
sub IsWLSProcessRunning
{
  #-----------------
  # Get Arguments...
  #-----------------
  my ( $sBEA_HOME,
       $sWlstScript, 
       $sEntryPointName,
       $sAdminURL,
       $sDomainName,
       $sDomainDir,
       $sServerName,
       $sServerRole,
       $sHost,
       $sPort,
       $snmHost,
       $snmPort,
       $iState,
       $iIState,
       $riPID ) = @_ ;

  #------------------
  # Announce Entry...
  #------------------
  my $sMyName = ( caller 0 )[3];
  &logTrace( 20133, "Subroutine <%s> called with 
  BEA_HOME                 [%s] 
  WlstScript                  [%s]
  EntryPointName           [%s] 
  AdminURL                 [%s]
  DomainName               [%s] 
  DomainDir                [%s]
  ServerName               [%s] 
  ServerRole               [%s] 
  Host                     [%s] 
  Port                     [%s] 
  nmHost                   [%s] 
  nmPort                   [%s] 
  State                    [%s] 
  IState                   [%s]", 
  $sMyName, $sBEA_HOME, $sWlstScript, $sEntryPointName, $sAdminURL,
  $sDomainName, $sDomainDir, $sServerName, $sServerRole, $sHost, $sPort, $snmHost,
  $snmPort, $iState, $iIState );

  my @hProcListHash=();
  my $iProcessCount = Proc::GetLongProcessListHash ( \@hProcListHash ,
                                                     $bEnv);

  #-----------------------------------
  # Confirm that processes were found.
  #-----------------------------------
  if( $iProcessCount == 0 )
  {
    #----------------------------------------------------------------------
    # Log error and return unknown - if we cant get a process list, we
    # certainly don't know the state.
    #----------------------------------------------------------------------
    &logError( 20134, "<%s>:Call to get process list did not return any processes",$sMyName);
    return 0;
  }

  my $sOSType = Arch::GetOSType();
    
  my ($sProtocol, $sAddrPort) = $sAdminURL =~ /(\w+):\/\/(.*)/ ;

  my $sFilter = '';
  if ( $sServerRole eq "Administrative" )
  {
        $sFilter = "java.*Name=$sServerName\\b";
  }
  elsif ( $sServerRole eq "Managed" )
  {
    if ( Arch::HPUX_OS == $sOSType ) {
      $sFilter = "java.*(management.server=\\w+://$sAddrPort.*Name=$sServerName\\b)|(Name=$sServerName\\b.*management.server=\\w+://$sAddrPort)";
      if ( $::gbNMBased )
      {
        $sFilter = "java.*Name=$sServerName\\b.*management.server=\\w+://$sAddrPort\\b";
      }
    }
    elsif ( Arch::AIX_OS == $sOSType ) {
      $sFilter = "java.*(management.server=\\w+://$sAddrPort.*Name=$sServerName\\b)|(Name=$sServerName\\b.*management.server=\\w+://$sAddrPort)";
      if ( $::gbNMBased )
      {
        $sFilter = "java.*Name=$sServerName\\b.*management.server=\\w+://$sAddrPort.*$sBEA_HOME.*$sDomainName";
      }
    }
    else {
      $sFilter = "java.*(management.server=\\w+://$sAddrPort.*Name=$sServerName\\b)|(Name=$sServerName\\b.*management.server=\\w+://$sAddrPort)";
    }
  }
  elsif ( $sServerRole eq "NodeManager" )
  {
    if ( Arch::AIX_OS == $sOSType ) {
      $sFilter = "java.*ListenAddress=$snmHost\\b.*ListenPort=$snmPort\\b";
    }
    else    { 
       $sFilter = "java.*ListenAddress=$snmHost\\b.*ListenPort=$snmPort\\b";
    }
    
  }

  my @hProcListHashMatched=();
  my $iNumberOfMatches = Proc::FilterProcs($sFilter, \@hProcListHash, \@hProcListHashMatched);

  if ( 0 < $iNumberOfMatches && $sServerRole eq "Administrative" ) {

      #---------------------------------------------------
      # First Check for DOMAIN_HOME Env in ps output
      #---------------------------------------------------

      my ($bDomEnvCheck, $bDomCheck, @hDomProcMatched ) = ();
      $bDomEnvCheck = 0;
      $bDomCheck    = 0;
      if ( $bEnv ) {
         foreach my $sProcLine (@hProcListHashMatched) {
            if ( $$sProcLine{"COMMAND"} =~ /\bDOMAIN_HOME=(\S+)/ ) {
               $bDomEnvCheck = 1;
               if ( $1 eq $sDomainDir ) {
                   &logTrace( 20135,"<%s>:DOMAIN_HOME Env found for Current Domain Dir [%s]",$sMyName, $1 );
                   push ( @hDomProcMatched, $sProcLine);
               }
               else {
                  &logTrace( 20136,"<%s>:DOMAIN_HOME Env [%s]found for Other Domain Dir [%s]",$sMyName, $1, $sDomainDir );
               }
            }
         }
      }

      if ( 1 == $bDomEnvCheck ){
          $iNumberOfMatches = scalar ( @hDomProcMatched );
          @hProcListHashMatched = @hDomProcMatched;
      }

      #---------------------------------------------------
      # If no match found above then 
      # Check for wl.domain=$sDomainName in ps output
      #---------------------------------------------------
     
      if ( ( 1 == $bDomEnvCheck && 0 == $iNumberOfMatches ) || ( 0 == $bDomEnvCheck && 0 < $iNumberOfMatches ) ) {
         foreach my $sProcLine (@hProcListHashMatched) {
            if ( $$sProcLine{"COMMAND"} =~ /\b[dD]omain=(\S+)/ ) {
                #----------------------------------------------
                # We are Strict if we got domain= in ps output
                #----------------------------------------------
                $bDomCheck = 1;
                if ( $1 eq $sDomainName ) {
                   &logTrace( 20137,"<%s>:Domain Name found in process list for Current Domain [%s]",$sMyName, $1 );
                   push ( @hDomProcMatched, $sProcLine);
                }
                else {
                   &logTrace( 20138,"<%s>:Domain Name found in process list for Other Domain [%s]",$sMyName, $1 );
                }
             }
          }

          if ( 1 == $bDomCheck ){
              $iNumberOfMatches = scalar ( @hDomProcMatched );
              @hProcListHashMatched = @hDomProcMatched;
          }
      }
  }

  &logTrace( 20139, "<%s>:Matches for processes with filter [%s] got [%s] matches",
                     $sMyName,
                     $sFilter,
                     $iNumberOfMatches);

  if( $iNumberOfMatches < 1 )
  {

    #---------------------------------------------------------------------------
    # If we are in the monitor entry point, then account for transitional states
    # and produce helpful log entries for resources found with no WLS processes
    # found.
    #---------------------------------------------------------------------------
    if( "monitor" eq $sEntryPointName )
    {
      #----------------------------------------------------------------------------
      # If the current resource State is (a kind of) OFFLINE, but we were trying to
      # go online (in one of it's various forms), and we found no WLS processes
      # for this resource, then produce an appropriate error message.
      #----------------------------------------------------------------------------
      if( ( VCSDef_ResourceState::OFFLINE               == $iState ||
            VCSDef_ResourceState::OFFLINE_STATE_UNKNOWN == $iState ||
            VCSDef_ResourceState::FAULTED_RESTARTING    == $iState
          ) && (
            VCSDef_ResourceIState::WAITING_TO_GO_ONLINE                   == $iIState ||
            VCSDef_ResourceIState::WAITING_TO_GO_ONLINE_REVERSE           == $iIState ||
            VCSDef_ResourceIState::WAITING_TO_GO_ONLINE_REVERSE_PROPAGATE == $iIState ||
            VCSDef_ResourceIState::WAITING_TO_GO_ONLINE_REVERSE_PATH      == $iIState ) )
      {
        &logError( 20140, "<%s>:Waiting to go online but no WebLogic Server processes found for this resource!",$sMyName);
        return 0;
      }
    }
    return 0;
  }
  else
  {
    my @liPID    = ();
    my $iCounter = 0;
    while ( $iCounter <  $iNumberOfMatches )
    {
      push(@liPID,$hProcListHashMatched[$iCounter]->{PID});
      &logTrace( 20141, "<%s>:Pushing pid [%s] for this WebLogic Server.",
                       $sMyName,$hProcListHashMatched[$iCounter]->{PID});
      $iCounter++;
    }
    $$riPID = $hProcListHashMatched[0]->{PID};
    &logTrace( 20142, "<%s>:Found one or more match in the Process table for this WebLogic Server.",
                       $sMyName);
    return 1;
  }
}


=pod

 -------------------------------------------------------------------------

 Function : int CheckWLSProcessID   ( IN  String  BEA_HOME,
                                      IN  String  WlstScript,
                                      IN  String  EntryPointName,
                                      IN  String  AdminURL,
                                      IN  String  DomainName,
                                      IN  String  DomainDir, 
                                      IN  String  ServerName,
                                      IN  String  ServerRole,
                                      IN  String  Host,
                                      IN  Integer Port,
                                      IN  String  nmHost,
                                      IN  Integer nmPort,
                                      IN  Integer State,
                                      IN  Integer IState,
                                      OUT Integer riPID )

 Description : Check whether the WLS processes are running on the system.
 This subroutine traces the process  id  passed as the last argument  and
 checks it  against the "ps -p" unix  behaviour, to determine whether the
 process is running. Then  it uses the following regex  to determine whether the
 it is the actual server process associated with that pid.

 It also detects if the pid file is stale, and  updates the  pidfile with
 the correct pid.

 AdminServer:  "^$sBEA_HOME.*java.*Name=$sServerName\\b.*$sDomainName.*Server\\b";
 Managed    :  "^$sBEA_HOME.*java.*Name=$sServerName\\b.*management.server=$sAdminURL.*$sDomainName";
 NodeManager:  "^$sBEA_HOME.*java.*ListenAddress=$snmHost\\b.*ListenPort=$snmPort\\b";

 Parameters :

  $sBEA_HOME              BEA Home Directory
  $sWlstScript               Product Installation Directory
  $sEntryPointName        Entry Point Name
  $sAdminURL              Administrative Server's URL
  $sDomainName            WLS Domain Name
  $sDomainDir             WLS Domain Directory
  $sServerName            WLS Server Name
  $sServerRole            WLS Server Role
  $sHost                  Listen Address of WLS Server
  $sPort                  Listen Port of Node manager
  $snmHost                ListenAddress of Node Manager
  $snmPort                Port of Node manager
  $iState                 State variable
  $iIState                IState variable
  $riPID                  Process id

 Returns : Integer value as defined below:

 0 Failure         WLS Process not running for this resource.
 1 Success         WLS Process running for this resource.

 -------------------------------------------------------------------------

=cut

sub CheckWLSProcessID
{
  #-----------------
  # Get Arguments...
  #-----------------
  my ( $sBEA_HOME,
       $sWlstScript, 
       $sEntryPointName,
       $sAdminURL,
       $sDomainName,
       $sDomainDir, 
       $sServerName,
       $sServerRole,
       $sHost,
       $sPort,
       $snmHost,
       $snmPort,
       $iState,
       $iIState,
       $iPID ) = @_ ;

  my $sMyName = ( caller 0 )[3];

  #------------------
  # Announce Entry...
  #------------------
  &logTrace( 20143, "Subroutine <%s> called with 
  BEA_HOME                 [%s] 
  WlstScript                  [%s] 
  EntryPointName           [%s] 
  AdminURL                 [%s]
  DomainName               [%s]
  DomainDir                [%s] 
  ServerName               [%s] 
  ServerRole               [%s] 
  Host                     [%s] 
  Port                     [%s] 
  nmHost                   [%s] 
  nmPort                   [%s]
  State                    [%s]
  IState                   [%s]
  PID                      [%s]",
  $sMyName, $sBEA_HOME, $sWlstScript, $sEntryPointName, $sAdminURL,
  $sDomainName, $sDomainDir, $sServerName, $sServerRole, $sHost, $sPort, $snmHost,
  $snmPort, $iState, $iIState, $iPID);

  my $sOSType = Arch::GetOSType();

  my $sFilter;
  if ( $sServerRole eq "Administrative" )
  {
      $sFilter = "java.*Name=$sServerName\\b";
  }
  elsif ( $sServerRole eq "Managed" )
  {
    if ( Arch::HPUX_OS == $sOSType ) {
    $sFilter = "java.*Name=$sServerName\\b.*managementServer=$sAdminURL\\b";
    }
    elsif ( Arch::AIX_OS == $sOSType ) {
      $sFilter = "java.*Name=$sServerName\\b.*management.server=$sAdminURL.*$sBEA_HOME.*$sDomainName";
    }
    else {
    $sFilter = "java.*Name=$sServerName\\b.*managementServer=$sAdminURL.*$sDomainName";
    }
  }
  elsif ( $sServerRole eq "NodeManager" )
  {
    if ( Arch::HPUX_OS == $sOSType ) {
      $sFilter = "java.*ListenAddress=$snmHost\\b.*ListenPort=$snmPort\\b";
    }
    elsif ( Arch::AIX_OS == $sOSType ) {
      $sFilter = "java.*ListenAddress=$snmHost\\b.*ListenPort=$snmPort\\b";
    }
    else {
      $sFilter = "java.*ListenAddress=$snmHost\\b.*ListenPort=$snmPort\\b";
    }
  }

  &logTrace(20144,"<%s>:Filter set to [%s]",$sMyName,$sFilter);

  #-----------------------------------------------
  # Set up correct ps command with arguments.
  # This need to be handled differently for each
  # platform.
  #-----------------------------------------------
    
  my $sCommandPs = "/usr/ucb/ps";
  my $sCommandArgsPs = "-ww";
  if ( Arch::HPUX_OS == $sOSType )
  {
    $sCommandPs     = "/bin/ps";
    $ENV{"UNIX95"}  = 1;
    $sCommandArgsPs = "-xxo 'uid,pid,state,ppid,args' -p";
  }
  elsif   (  Arch::LINUX_OS == $sOSType
          || Arch::AIX_OS   == $sOSType )
  {
    $sCommandPs = "/bin/ps";
    if ( Arch::LINUX_OS == $sOSType )
    {
      $sCommandArgsPs = "-wwo 'uid,pid,state,ppid,args' -p";
    }
    else
    {
      $sCommandArgsPs = "-o 'uid,pid,state,ppid,args' -p";
    }
  }
  $sCommandArgsPs = $sCommandArgsPs." ".$iPID;
  #---------------------------------------------------------
  # SUN   : [/usr/ucb/ps -ww $iPID]
  # LINUX : [/bin/ps -o 'uid,pid,state,ppid,args' -p $iPID]
  # AIX   : [/bin/ps -o 'uid,pid,state,ppid,args' -p $iPID]
  # HP    : [/bin/ps -xxo 'uid,pid,state,ppid,args' -p $iPID]
  #---------------------------------------------------------
    
  my $sResourceType   = $ENV{VCS_LOG_AGENT_NAME};
  my $sResourceName   = $ENV{VCS_LOG_RESOURCE_NAME};
  my $sPidFile        = "/tmp/.VRTS$sResourceType/$sResourceName.pid";

  my $sEnvFile       = "";
  my $sUser          = "root";
  my $iTimeOut       = 5;
  my $iExitCode;
  my @lsCommandOut;
  my $bTimedOut;

  my $iReturnCode = Sys::RunWithEnvCmdWithOutputWithTimeOut($sEnvFile,
                                                           DONT_SOURCE_USER_ENV,
                                                           $sCommandPs,
                                                           $sCommandArgsPs,
                                                           \$iExitCode,
                                                           $sUser,
                                                           \@lsCommandOut,
                                                           \$bTimedOut,
                                                           $iTimeOut);
  if ( $iReturnCode )
  {
    #---------------------------
    # Command got executed.
    #---------------------------
    if ( $iExitCode )
    {
      #---------------------
      # Command failed !
      #---------------------
      &logTrace (20145,"<%s>:command [%s] with args [%s] failed with exit code [%s]",
                 $sMyName,$sCommandPs,$sCommandArgsPs,$iExitCode);

      #-------------------------------------------------------------------
      # Command may fail if the process is not running hence we need to 
      # again confirm whether other pid is running for this server.
      #-------------------------------------------------------------------
      my $iReturn = IsWLSProcessRunning($sBEA_HOME ,
                                        $sWlstScript, 
                                        $sEntryPointName,
                                        $sAdminURL,
                                        $sDomainName,
                                        $sDomainDir,
                                        $sServerName,
                                        $sServerRole,
                                        $sHost,
                                        $sPort,
                                        $snmHost,
                                        $snmPort,
                                        $iState,
                                        $iIState,
                                        \$iPID );
      if ( 1 == $iReturn )
      {
        #--------------------------------------------------
        # Got valid process running for this WLS instance.
        # This means the pid file is stale.
        #--------------------------------------------------
        &logWarn(20146,"<%s>:Pidfile [%s] is stale.",$sMyName,$sPidFile);

        $iReturn = open (FH,"> $sPidFile");
         if ( ! $iReturn )
         {
          &logError( 20147, "<%s>:Cannot open file [%s] for writing.",$sMyName,$sPidFile);
         }
         else
         {
          &logInfo( 20148, "<%s>: Updated PidFile [%s] with [%s].",$sMyName,$sPidFile,$iPID);
           print ( FH  $iPID);
           close FH;
         }
        return 3;
      }
      return 0;
    }
    else
    {
      #------------------------
      # Successful execution.
      #------------------------
      &logTrace(20149,"<%s>:command [%s] with args [%s] successfully executed with exit code [%s]",
                           $sMyName,$sCommandPs,$sCommandArgsPs,$iExitCode);


      #--------------------------------------------------------------
      # Now we need to check whether this pid is actually our server !
      #--------------------------------------------------------------
      my $sLine;
      foreach $sLine (@lsCommandOut)
      {
        if ( $sLine =~ /$sFilter/ )
        {
          &logTrace(20150,"<%s>:WLS process found [%s]",$sMyName,$sLine);
          my (@lsEntry) = split " " ,$sLine;
          if ( $lsEntry[2] eq "Z" )
          {
            #----------------------
            # Process is zombie !
            #----------------------
            &logWarn(20151,"<%s>:Process State [%s] is zombie.",
                     $sMyName,$lsEntry[2]);
            return 2;
          }

          #------------------------------------------
          # Got proper process running for this pid.
          #------------------------------------------
          return 1;
        }
      }

      #-----------------------------------------------------------------------
      # We reached here because matching process not found with the given pid.
      # Now we have to check whether process is running with other pid.
      #-----------------------------------------------------------------------

      &logTrace(20152,"<%s>:matching process [%s] not found from pid file.",$sMyName,$iPID);
      &logTrace(20153,"<%s>:Filter used [%s].",$sMyName,$sFilter);
      my $iReturn = IsWLSProcessRunning($sBEA_HOME ,
                                        $sWlstScript, 
                                        $sEntryPointName,
                                        $sAdminURL,
                                        $sDomainName,
                                        $sDomainDir,
                                        $sServerName,
                                        $sServerRole,
                                        $sHost,
                                        $sPort,
                                        $snmHost,
                                        $snmPort,
                                        $iState,
                                        $iIState,
                                        \$iPID );
      if ( 1 == $iReturn )
      {
        #--------------------------------------------------
        # Got valid process running for this WLS instance.
        # This means the pid file is stale.
        #--------------------------------------------------
        &logWarn(20154,"<%s>:Pidfile [%s] is stale.",$sMyName,$sPidFile);

        $iReturn = open (FH,"> $sPidFile");
         if ( ! $iReturn )
         {
          &logError( 20155, "<%s>:Cannot open file [%s] for writing.",$sMyName,$sPidFile);
         }
         else
         {
          &logInfo( 20156, "<%s>:Updated PidFile [%s] with [%s].",$sMyName,$sPidFile,$iPID);
           print ( FH  $iPID);
           close FH;
         }
        return 3;
      }
      return 0;
    }
  }
  else
  {
    if ( $bTimedOut )
    {
      #--------------------
      # Command timed out.
      #--------------------
      &logError (20157,"<%s>:Command [%s] with args [%s] did not complete within [%s] seconds.",
      $sMyName,$sCommandPs,$sCommandArgsPs,$iTimeOut);
    }
    else
    {
      #--------------------
      # Unknown error.
      #--------------------
      &logError(20158,"<%s>:Command [%s] with args [%s] execution is unknown.",$sMyName,$sCommandPs,$sCommandArgsPs);
    }
    return 0;
  }
}

=pod

 -------------------------------------------------------------------------

 Function : String, Integer ExtractIP( IN  String  HostPort )

 Description : Extracts the IP and Port from a <hostname:portnum> string.
 The subroutine does  a  inet_ntoa on  the hostname  and  fetches its IP.
 Returns the extracted IP and the Port if successful.

 Parameters :

  $sHostPort              String in <Host:Port> format.

 Returns : List values as defined below:

 (HostString , NumPort)    Extracted IP address and port from the
                           <host:port> string.
 Undef                     Unable to extract the IP

 -------------------------------------------------------------------------

=cut

sub ExtractIP
{
  my ($sHostPort) = @_;
  my $sResourceType   = $ENV{VCS_LOG_AGENT_NAME};
  my $sResourceName   = $ENV{'VCS_LOG_RESOURCE_NAME'};
  my $sEntryPointName = $ENV{'VCS_LOG_SCRIPT_NAME'};

  my $sMyName = (caller 0)[3];
  my $sIPDir  = "/tmp/.VRTS$sResourceType";
  my $sIPFile = "$sIPDir/$sResourceName.ip";

  &logTrace( 20159, "Subroutine <%s> called with: HostPort [%s]",
                   $sMyName, $sHostPort );
  
  my ($sIP,$sPort) = split(/:/,$sHostPort);

  # Check whether IP is in x.y.z.a number format 
  if ( $sIP =~ /[0-9]+\.[0-9]+\.[0-9]+\.[0-9]+/) 
  {
    #---------------------------------
    # Format is correct [12.22.23.43] 
    #---------------------------------
    return ($sIP, $sPort);
  }
  else 
  {
    my $sHostName;

    if ( "online" eq $sEntryPointName )
    {
      # create file
      
      unlink ( $sIPFile );
      $sHostName = gethostbyname($sIP);
      if ( $sHostName )
      {
        $sIP = Socket::inet_ntoa($sHostName); 
        &logTrace(20160, "<%s>:Returning IP [%s] and Port [%s]",
                                      $sMyName, $sIP , $sPort);
      }
      else
      {
        #---------------------------------------------
        # Log error as the gethostbyname call failed.
        # dont return anything.
        #---------------------------------------------
        &logTrace(20161, "<%s>:gethostbyname failed for [%s]",
                                      $sMyName, $sIP);
        return ;
      }
      if ( -d $sIPDir )
      {
        &logTrace ( 20162, "<%s>:IP folder [%s] exists", $sMyName, $sIPDir );
      }
      else
      {
        mkdir ( $sIPDir, 0777 ) ||
        &logWarn ( 20163, "<%s>:Failed to create IP folder [%s]",
        $sMyName, $sIPDir );
        chmod (0777,$sIPDir );
      }

      #--------------------------
      # Create IP history file..
      #--------------------------
      if ( sysopen ( IPFILE, $sIPFile, O_RDWR|O_SYNC|O_CREAT, 0600 ) ) {
        &logTrace ( 20164,
        "<%s>:Opened IP file [%s] successfully for read/write",
        $sMyName, $sIPFile );
      }
      else {
        &logWarn ( 20165,
        "<%s>:Failed to open IP file [%s] for read/write",
        $sMyName, $sIPFile );
      }
      print IPFILE "$sIP\n";
      close IPFILE;
      return ($sIP,$sPort);
    }
    elsif ( "monitor" eq $sEntryPointName )
    {
      if (! -f $sIPFile )
      {
        $sHostName = gethostbyname($sIP);
        if ( $sHostName )
        {
          $sIP = Socket::inet_ntoa($sHostName);
          &logTrace(20166, "<%s>:Returning IP [%s] and Port [%s]",
                                        $sMyName, $sIP , $sPort);
        }
        else
        {
          #---------------------------------------------
          # Log error as the gethostbyname call failed.
          # dont return anything.
          #---------------------------------------------
          &logTrace(20167, "<%s>:gethostbyname failed for [%s]",
                                        $sMyName, $sIP);
          return ;
        }
        if ( -d $sIPDir )
        {
          &logTrace ( 20168, "<%s>:IP folder [%s] exists", $sMyName, $sIPDir );
        }
        else
        {
          mkdir ( $sIPDir, 0777 ) ||
          &logWarn ( 20169, "<%s>:Failed to create IP folder [%s]",
          $sMyName, $sIPDir );
          chmod (0777,$sIPDir );
        }

        #--------------------------
        # Create IP history file..
        #--------------------------
        if ( sysopen ( IPFILE, $sIPFile, O_RDWR|O_SYNC|O_CREAT, 0600 ) ) {
          &logTrace ( 20170,
          "<%s>:Opened IP file [%s] successfully for read/write",
          $sMyName, $sIPFile );
        }
        else {
          &logWarn ( 20171,
          "<%s>:Failed to open IP file [%s] for read/write",
          $sMyName, $sIPFile );
        }
        print IPFILE "$sIP\n";
        close IPFILE;
        return ($sIP,$sPort);
      }
      else
      {
        #---------------
        # Read in file..
        #---------------
        if ( sysopen ( IPFILE, $sIPFile, O_RDWR|O_SYNC|O_CREAT, 0600 ) ) {
          &logTrace ( 20172,
          "<%s>:Opened IP file [%s] successfully for read/write",
          $sMyName, $sIPFile );
        }
        else {
          &logWarn ( 20173,
          "<%s>:Failed to open IP file [%s] for read/write",
          $sMyName, $sIPFile );
        }

        ( $sIP ) = grep { s/\n// } <IPFILE>;
        return ($sIP,$sPort);
      }
    }
    else #offline (clean will not come here)
    {
      $sHostName = gethostbyname($sIP);
      if ( $sHostName ) 
      {
        $sIP = Socket::inet_ntoa($sHostName);
        &logTrace(20174, "<%s>:Returning IP [%s] and Port [%s]",
                                      $sMyName, $sIP , $sPort);
        return ($sIP,$sPort);
      }
      else 
      {
        #---------------------------------------------
        # Log error as the gethostbyname call failed.
        # dont return anything.
        #---------------------------------------------
        &logTrace(20175, "<%s>:gethostbyname failed for [%s]",
                                      $sMyName, $sIP);
        return ;
      }
    }
  }
  return ;
}

=pod

 -------------------------------------------------------------------------

 Function : Integer CheckIfIPExistsLocally ( IN  String  IP )

 Description : This subroutine determines whether the  IP passed as 
 argument is plumbed on this node. It uses ifconfig -a/netstat -in.

 Parameters :

  $sHostPort              String in <Host:Port> format.

 Returns : Integer value as defined below:

 0 Failure         IP is plumbed on this node.
 1 Success         IP is NOT plumbed on this node.

 -------------------------------------------------------------------------

=cut

sub CheckIfIPExistsLocally
{
  my ( $sIP ) = @_;

  my $sMyName = (caller 0)[3];

  &logTrace( 20176, "Subroutine <%s> called with: IP [%s]",
                   $sMyName, $sIP );

  my $sCmdIfConfig = "/sbin/ifconfig";
  my $sCmdNetstat  = "/bin/netstat";
  my $sCmdGrep     = "/bin/grep";
  my $sOSType      = Arch::GetOSType();

  my $iIPMatchCount = 0;

  return $iIPMatchCount unless ( $sIP =~ /\S/ );

  if ( Arch::HPUX_OS != $sOSType ) 
  {
    if ( Arch::AIX_OS   == $sOSType ) 
    {
      $sCmdIfConfig = "/usr/sbin/ifconfig"; 
    }
    $iIPMatchCount = `$sCmdIfConfig -a | $sCmdGrep -cw "$sIP"`;
  }
  else 
  {
    # HP-UX do 'netstat -in'
    $iIPMatchCount = `$sCmdNetstat -in | grep -cw "$sIP"`;
  }

  $iIPMatchCount ||= 0; 
  if( $iIPMatchCount > 0) 
  {
    &logTrace ( 20177, "<%s>:IP [%s] is plumbed.", $sMyName, $sIP );
    return 1;
  }
  else 
  {
    &logTrace ( 20178, "<%s>:IP [%s] is not plumbed.", $sMyName, $sIP );
    return 0;
  }
}

=pod

 -------------------------------------------------------------------------

 Function : List String ParseURL ( IN  String  IP )

 Description : Parse the URL from "http://<hostname>:<portnum>" format.
 It returns a list of string with first element as the hostname and
 second as portnum.

 Parameters :

  $sURL              String in <http://Host:Port> format.

 Returns : List value as defined below:

 (protocol,host,port) Success 
 Undef                Failure 

 -------------------------------------------------------------------------

=cut

sub ParseURL
{
  my ( $sURL ) = @_;

  #---------------------------------------------------
  # Attempt to parse.  Return undef if it didn't parse
  #---------------------------------------------------
  (my @parsed =$sURL =~ m@(\w+)://([^/:]+)(:\d*)?([^#]*)@) || return undef;

  #------------------------------
  # remove colon from port number
  #------------------------------
  if( defined $parsed[2] )
  {
     $parsed[2]=~ s/^://;
  }
  else
  {
    return undef;
  }
  return @parsed;
}

=pod

 -------------------------------------------------------------------------

 Function : Integer GetWLSProcessRunTime ( IN  Integer  PID )

 Description : Determines how many seconds the process is been running.
 It uses the "etime" option of "ps" command.

 Parameters :

  $iPID              Process id for the WebLogic Server process

 Returns:

 Positive Integer      Number of seconds the process is running.
 -1                    If not able to determine the process run time.

 -------------------------------------------------------------------------

=cut

sub GetWLSProcessRunTime
{
  my ( $iPID ) = @_;

  my $sMyName = ( caller 0)[3];
  &logTrace ( 20179, "Subroutine <%s> called with [%s].", $sMyName, $iPID);
  
  #-----------------------------------------------
  # Set up correct ps command with arguments.
  # This need to be handled differently for each
  # platform.
  #-----------------------------------------------
  my $sOSType = Arch::GetOSType();

  my $sCommandPs = "/bin/ps";
  my $sCommandArgsPs = "";

  if ( Arch::HPUX_OS == $sOSType )
  {
    $sCommandPs     = "/bin/ps";
    $ENV{"UNIX95"}  = 1;
  }
  elsif   (  Arch::LINUX_OS == $sOSType
          || Arch::AIX_OS   == $sOSType )
  {
    $sCommandPs = "/bin/ps";
  }
  $sCommandArgsPs = "-p ".$iPID." -o etime";

  my $sUser          = "root";
  my $sEnvFile       = "";
  my $iTimeOut       = 10;
  my $iExitCode;
  my $bTimedOut;
  my @lsCommandOut;
  
  my $iReturnCode = Sys::RunWithEnvCmdWithOutputWithTimeOut($sEnvFile,
                                                           DONT_SOURCE_USER_ENV,
                                                           $sCommandPs,
                                                           $sCommandArgsPs,
                                                           \$iExitCode,
                                                           $sUser,
                                                           \@lsCommandOut,
                                                           \$bTimedOut,
                                                           $iTimeOut);
  if ( $iReturnCode )
  {
    #---------------------------
    # Command got executed.
    #---------------------------
    if ( $iExitCode )
    {
      #---------------------
      # Command failed !
      #---------------------
      &logTrace (20180,"<%s>:Command [%s] with args [%s] failed with exit code [%s]",
                 $sMyName,$sCommandPs,$sCommandArgsPs,$iExitCode);
      return;
    }
    else
    {
      #------------------------
      # Successful execution.
      #------------------------
      &logTrace(20181,"<%s>:Command [%s] with args [%s] successfully executed with exit code [%s]",
                           $sMyName,$sCommandPs,$sCommandArgsPs,$iExitCode);
      my $iElapsedSeconds = 0;
      my ($sEDay, $sEHour, $sEMin, $sESec) = $lsCommandOut[1] =~ /(\d*-)?(\d*:)?(\d*):(\d*)$/;

      my $iSecondsRun = 0 ;
      if ( $sEDay )
      {
        $sEDay =~ s/-//g;
        $iSecondsRun += ($sEDay * 60 * 60 * 24);
      }
      if ( $sEHour )
      {
        $sEHour =~ s/://g;
        $iSecondsRun += ($sEHour * 60 * 60);
      }
      if ( $sEMin )
      {
        $iSecondsRun += ($sEMin * 60);
      }
      if ( $sESec )
      {
        $iSecondsRun += $sESec ;
      }
      &logTrace(20182, "<%s>:Process running for [%s] seconds.",$sMyName, $iSecondsRun);
      return $iSecondsRun;
    }
  }
  else
  {
    if ( $bTimedOut )
    {
      #--------------------
      # Command timed out.
      #--------------------
      &logError (20183,"<%s>:Command [%s] with args [%s] did not complete within [%s] seconds.",
      $sMyName,$sCommandPs,$sCommandArgsPs,$iTimeOut);
      &logTrace( 20184,"<%s>:Sleeping till had kills the entry point.",$sMyName);
      sleep (1000) while(1);
    }
    else
    {
      #--------------------
      # Unknown error.
      #--------------------
      &logError(20185,"<%s>:Command [%s] with args [%s] execution is unknown.",
                       $sMyName,$sCommandPs,$sCommandArgsPs);
    }
  }
  return -1;
}

=pod
 -------------------------------------------------------------------------
 Function : int GetEPTimeout ( IN String EntryPointName
                               IN String ResourceType  );

 Description : Returns the Entry Point Timeout for the EP which we are in

 Parameters :

 $sEntryPointName           The EP which we are currently running
 $sResourceType             VCS resource type

 Returns  : Integer value as defined below:

 -------------------------------------------------------------------------
=cut

sub GetEPTimeout
{
  my ( $sEntryPointName, $sResourceType) = @_;

  #------------------
  # Announce Entry..
  #------------------

  my $sMyName = ( caller 0 )[3];
  &logTrace( 20186, "Subroutine <%s> called with:
  EntryPointName [%s]
  ResourceType   [%s]", 
  $sMyName , $sEntryPointName , $sResourceType );

  my $sObjectName          = "hatype";
  my %hTypeValues;

  my $iEngineTimeout;
  #-----------------------------------
  # Query if it has already been set..
  #-----------------------------------
  if ( defined $::giEPTimeout &&    # Defined..
       $::giEPTimeout =~ /^\d+/ ) { # to an integer..
    $iEngineTimeout = $::giEPTimeout;

    &logTrace ( 20187,
    "<%s> Using AEPTimeout [%s] as EP timeout value",
    $sMyName, $iEngineTimeout );
    return ( $iEngineTimeout ); 
  }

  if ( ! VCSagentFW::GetVCSObjectHash($sObjectName,$sResourceType,\%hTypeValues) )
  {
    &logError(20188,"<%s>:GetVCSObjectHash call failed.",$sMyName);
    return 0;
  }

  #-------------------------------
  # Make "monitor" to "Monitor"
  # offline to Offline and so on
  #-------------------------------

  my $sTempEntry =  $sEntryPointName ;
  $sTempEntry    =~ s/(\w)(.*)$/\U$1\L$2/; 

  &logTrace(20189,"<%s>: Returning [%s] value [%s]",$sMyName, $sTempEntry."Timeout",$hTypeValues{$sTempEntry."Timeout"});
  return ( $hTypeValues{$sTempEntry."Timeout"});
}

=pod

 -------------------------------------------------------------------------
 Function : int DelayEntryPoint ( IN Integer DelayToTime
                                  IN Integer State ,
                                  IN Integer IState ,
                                  IN String  User , 
                                  IN String  WLSUser , 
                                  IN String  WLSPassword , 
                                  IN String  nmUser , 
                                  IN String  nmPassword , 
                                  IN String  BEA_HOME,
                                  IN String  WlstScript,
                                  IN String  Host
                                  IN Integer Port
                                  IN String  nmHost
                                  IN Integer nmPort
                                  IN String  ServerName , 
                                  IN String  ServerRole , 
                                  IN String  DomainName , 
                                  IN String  DomainDir,  
                                  IN String  AdminURL , 
                                  IN Integer SecondLevelMonitor , 
                                  IN String  nmType ,
                                  IN String  MonitorProgram )

 Description :   The  DelayEntryPoint  function  suspends the  logical 
 execution of the  agent  until either a condition is met , or  it has
 reached  the  ExpirationTime  passed to  it. It  will  test  for  the
 condition every  5  seconds until  it is met , or time  has  run out, 
 whichever occurs first.  

 In online entry point this subroutine delays the entry point till the 
 server responds on (nm)ListenAddressPort. Further  it also delays the
 online entry point till the server  also starts  responding to second
 level monitoring checks if specified.

 In monitor entry point this subroutine delays the entry point till the
 server starts responding at ListenAddressPort , if the WLS server gets
 restarted  by  NodeManager. Thus  it  allows sufficient  time  for the
 restarted WebLogic Server to fully initialize.

 In offline entry point this subroutine delays the entry point till the
 WebLogic server process stops running.

 Parameters :

  $iDelayToTime           Time till the entry point could be delayed
  $iState                 State variable
  $iIState                IState variable
  $sUser                  Unix User
  $sWLSUser               WebLogic Server User
  $sWLSPassword           WebLogic Server Password
  $sBEA_HOME              BEA Home Directory
  $sWlstScript               Product Installation Directory
  $sHost                  ListenAddress of the WLS Server
  $sPort                  Port of the WLS Server
  $snmHost                ListenAddress of the NodeManager Server
  $snmPort                Port of the NodeManager
  $sServerName            WLS Server Name
  $sServerRole            WLS Server Role
  $sDomainName            WebLogic Domain Name
  $sDomainDir             WebLogic Domain Directory
  $sAdminURL              Administrative Server's URL
  $iSecondLevelMonitor    Second Level Monitor
  $snmType                Node Manager Type
  $sMonitorProgram        External Monitor Program

 Returns:
 
  0 Failure         Failure in delaying the entry point.
  1 Success         Successfully delayed the entry point.

 -------------------------------------------------------------------------

=cut

sub DelayEntryPoint
{
  my ( $iDelayToTime,
       $iState,
       $iIState,
       $sUser,
       $sWLSUser,
       $sWLSPassword,
       $snmUser,
       $snmPassword,
       $sBEA_HOME,
       $sWlstScript,
       $sHost,
       $sPort,
       $snmHost,
       $snmPort,
       $sServerName,
       $sServerRole,
       $sDomainName,
       $sDomainDir,  
       $sAdminURL,
       $iSecondLevelMonitor,
       $snmType,
       $sMonitorProgram ) = @_;

  my $sMyName         = ( caller 0)[3];
  my $iLoopCounter    = 1;
  my $sEntryPointName = $ENV{'VCS_LOG_SCRIPT_NAME'};

  &logTrace ( 20190, "Subroutine <%s> called with:
  EntryPointName           [%s]
  DelayToTime              [%s]
  State                    [%s]
  IState                   [%s]
  User                     [%s]
  WLSUser                  [%s]
  WLSPassword              [%s]
  nmUser                   [%s]
  nmPassword               [%s]
  BEA_HOME                 [%s]
  WlstScript               [%s]
  Host                     [%s]
  Port                     [%s]
  ServerName               [%s]
  ServerRole               [%s]
  DomainName               [%s]
  DomainDir                [%s] 
  AdminURL                 [%s]
  SecondLevelMonitor       [%s]
  nmType                   [%s]
  MonitorProgram           [%s]",

  $sMyName, $sEntryPointName, $iDelayToTime, 
  $iState,$iIState, $sUser, $sWLSUser, $sWLSPassword, $snmUser, $snmPassword, $sBEA_HOME, $sWlstScript, $sHost,
  $sPort, $sServerName, $sServerRole, $sDomainName, $sDomainDir, $sAdminURL,  
  $iSecondLevelMonitor, $snmType, $sMonitorProgram);

  my $iReturnCode = 0;
  my $iNSR        = 0;

  if ( "monitor" eq $sEntryPointName || "online" eq $sEntryPointName)
  {
    my $riPID;
    my $bFlag1 = 1;
    my $bFlag2 = 1;
    if ( "online" eq $sEntryPointName && $::gbNMBased )
    {
      sleep (15);
    }
    while ( $bFlag1 && $bFlag2 < 8 )
    {
      if ( WebLogic::IsWLSProcessRunning( $sBEA_HOME ,
                                         $sWlstScript,
                                         $sEntryPointName,
                                         $sAdminURL,
                                         $sDomainName,
                                         $sDomainDir, 
                                         $sServerName,
                                         $sServerRole,
                                         $sHost,
                                         $sPort,
                                         $snmHost,
                                         $snmPort,
                                         $iState,
                                         $iIState,
                                         \$riPID ) )
      {
        #----------------------------------------------------
        # Process is running.
        # Proceed to do first level socket connection check.
        #----------------------------------------------------
        &logTrace(20191,"<%s>:Server Proces is running with [%s] pid.",
                         $sMyName, $riPID);
        $bFlag1 = 0;
      }
      else
      {

        #----------------------------------------------------
        # Process is NOT running.
        # Proceed to do first level socket connection check.
        #----------------------------------------------------
        &logWarn(20192,"<%s>:Server Process is not running.", $sMyName);
        if ( $::gbNMBased )
        {
          return 0;
        }
        else 
        {
          &logTrace(20193,"<%s>:NON NodeManager configuration. Sleeping for 5 seconds", $sMyName);
          sleep (5);
          $bFlag2+=1;
        }
      }
    }

    #-----------------------------------------------------------------
    # We need to delay till first level socket connection succeeds .
    #-----------------------------------------------------------------

    #------------------------------
    # Number of Seconds Remaining.
    #------------------------------
    $iNSR = $iDelayToTime - time ;

    my $sCheckHost = $sHost ;
    my $sCheckPort = $sPort ;

    if ( "NodeManager" eq $sServerRole )
    {
      $sCheckHost = $snmHost;
      $sCheckPort = $snmPort;
    }

    &logInfo(20194, "<%s>: Server not responding for first level check at [%s:%s]. Waiting for Server to respond",
                        $sMyName,$sCheckHost,$sCheckPort );

    while( $iNSR > CHECK_INTERVAL )
    {
      if( $iNSR <= 5 )
      {
        &logInfo(20195, "<%s>:Not enough time left ([%s] seconds) to perform the WebLogic Server checks", $sMyName,$iNSR );
        if( $iNSR > 0 )
        {
          &logInfo(20196, "<%s>:Sleeping [%s] seconds", $sMyName, $iNSR );
          sleep $iNSR;
        }
        return 0;
      }

      if (  Comms::CheckPort($sCheckHost,$sCheckPort) )
      {
        &logInfo(20197, "<%s>: First Level Check , server responding at [%s:%s].", $sMyName,$sCheckHost,$sCheckPort );
        $iReturnCode = 1;
        last;
      }
      else
      {
        &logTrace(20198, "<%s>: Server not responding for first level check at [%s:%s]. Sleeping for [%s] seconds.", 
                        $sMyName,$sCheckHost,$sCheckPort,CHECK_INTERVAL );
        sleep CHECK_INTERVAL;
        $iNSR = $iDelayToTime - time ;
      }
    }

    if ( "monitor" eq $sEntryPointName ) {return $iReturnCode};

    #-----------------------------------------------------------------------
    # We are in "online" entry point.Need to allow the server to initialize 
    # till it responds to second level monitor if specified.
    #-----------------------------------------------------------------------
    if ( $iSecondLevelMonitor )
    {
      $iNSR = $iDelayToTime - time - EPOP;

      &logInfo(20199, "<%s>: Waiting for Server to respond for Second Level Monitor checks.", $sMyName );

      while( $iNSR > CHECK_INTERVAL )
      {
        if( $iNSR <= 5 )
        {
          &logInfo(20200, "<%s>:Not enough time left ([%s] seconds) to perform the WebLogic Server checks", $sMyName, $iNSR );
          if( $iNSR > 0 )
          {
            &logTrace(20201,"<%s>:Sleeping [%s] seconds", $sMyName, $iNSR );
            sleep $iNSR;
          }
          return $iReturnCode;
        }

        my $iTimeOut = $iNSR;
        if ( VCSDef_MonitorExit::ONLINE == SecondLevelMonitor ( $sUser,
                                                 $sWLSUser,
                                                 $sWLSPassword,
                                                 $snmUser,
                                                 $snmPassword,
                                                 $sCheckHost,
                                                 $sCheckPort,
                                                 $sServerRole,
                                                 $sDomainName,
                                                 $sDomainDir,
                                                 $sBEA_HOME,
                                                 $sWlstScript,
                                                 $snmType,
                                                 $sAdminURL,
                                                 $iTimeOut))
        {
          &logInfo(20202, "<%s>:Server [%s] responding to Second Level Monitor checks.", $sMyName,$sServerName);
          $iReturnCode = 1;
          last;
        }
        else
        {
          &logTrace(20203, "<%s>:Server not ready for SLM. Sleeping for [%s] seconds.", $sMyName,CHECK_INTERVAL );
          sleep CHECK_INTERVAL;
          $iNSR = $iDelayToTime - time ;
        }
      }
    }
  }
  else
  {
    #---------------------------------------------------------------------
    # We are in offline EP. Delay till all the running processes are gone.
    #---------------------------------------------------------------------

    #---------------------------------------------------------------------
    # Optimistic value of iReturnCode set to 0 ie. process not running.
    #---------------------------------------------------------------------
    $iReturnCode = 0;

    my $riPID;
    if ( WebLogic::IsWLSProcessRunning( $sBEA_HOME ,
                                       $sWlstScript,
                                       $sEntryPointName,
                                       $sAdminURL,
                                       $sDomainName,
                                       $sDomainDir,  
                                       $sServerName,
                                       $sServerRole,
                                       $sHost,
                                       $sPort,
                                       $snmHost,
                                       $snmPort,
                                       $iState,
                                       $iIState,
                                       \$riPID ) )
    {
      $iNSR = $iDelayToTime - time ;
      while( $iNSR > CHECK_INTERVAL )
      {
        if( $iNSR <= 5 )
        {
          &logInfo(20204, "<%s>:Not enough time left ([%s] seconds) to perform the WebLogic Server checks", $sMyName, $iNSR );
          if( $iNSR > 0 )
          {
            &logTrace(20205, "<%s>:Sleeping [%s] seconds", $sMyName, $iNSR );
            sleep $iNSR;
          }
          return 0;
        }
        if ( WebLogic::IsWLSProcessRunning( $sBEA_HOME ,
                                        $sWlstScript,
                                        $sEntryPointName,
                                        $sAdminURL,
                                        $sDomainName,
                                        $sDomainDir,
                                        $sServerName,
                                        $sServerRole,
                                        $sHost,
                                        $sPort,
                                        $snmHost,
                                        $snmPort,
                                        $iState,
                                        $iIState,
                                        \$riPID ) )
        {
          &logTrace(20206, "<%s>:Server process still running.", $sMyName);
          &logTrace(20207, "<%s>:Sleeping for [%s] seconds.", $sMyName,CHECK_INTERVAL );
          sleep CHECK_INTERVAL;
          $iNSR = $iDelayToTime - time ;
        }
        else
        {
          &logTrace(20208, "<%s>:Server processes not running.", $sMyName);
          $iReturnCode = 1;
          last;
        }
      }
    }
    else
    {
      #------------------------------------------------------------
      # Success for offline EP as the process is no longer running.
      #------------------------------------------------------------
      $iReturnCode = 1;
    }
  }

  return $iReturnCode;
}

=pod

 -------------------------------------------------------------------------
 Function : int SecondLevelMonitor ( IN String  User , 
                                     IN String  WLSUser , 
                                     IN String  WLSPassword , 
                                     IN String  nmUser , 
                                     IN String  nmPassword , 
                                     IN String  Host
                                     IN Integer Port
                                     IN String  ServerRole , 
                                     IN String  DomainName ,
                                     IN String  DomainDir, 
                                     IN String  BEA_HOME ,
                                     IN String  WlstScript,  
                                     IN String  nmType ,
                                     IN String  AdminURL ,
                                     IN Integer TimeOut )

 Description :  Second  Level monitoring checks for this WebLogic server
 resource. The  subroutine uses BEA's WebLogic  Scripting  Tool  utility 
 to do the  slm  checks.  In case  of  ServerRole being  NodeManager  it 
 connects to  the  NodeManager  via wlst.sh  using  the  nmConnect() api
 with correct arguments. If the connection via nmConnect() is successful
 the SLM  for NodeManager  succeeds. In  case of  Administrative/Managed
 ServerRole , it  connects  to  the  WLS  server via  wlst.sh  using the
 connect() api with correct arguments. If this connection  is successful
 the SLM succeeds.

 Parameters :

  $sUser                  Unix User
  $snmUser               SecondLevelMonitor User
  $snmPassword           SecondLevelMonitor Password
  $sHost                  ListenAddress of the WLS Server
  $sPort                  Port of the WLS Server
  $sServerRole            WLS Server Role
  $sDomainName            WLS Domain Name
  $sDomainDir             WLS Domain Directory  
  $sBEA_HOME              BEA Home Directory
  $sWlstScript            WLST Script Location
  $snmType                Node Manager Type
  $sAdminURL                Domain-wide Administration Mode State
  $iTimeOutProgram        Time out in seconds for second level monitor

 Returns : Integer value as defined below:

 99   UNKNOWN                          VCSDef_MonitorExit::UNKNOWN
 100  OFFLINE                          VCSDef_MonitorExit::OFFLINE
 110  ONLINE                           VCSDef_MonitorExit::ONLINE

 -------------------------------------------------------------------------

=cut

sub SecondLevelMonitor
{
  my ( $sUser,
       $sWLSUser,
       $sWLSPassword,
       $snmUser,
       $snmPassword,
       $sHost,
       $sPort,
       $sServerRole,
       $sDomainName,
       $sDomainDir,  
       $sBEA_HOME,
       $sWlstScript,
       $snmType,
       $sAdminURL,
       $iTimeOut) = @_;

  my $sMyName = ( caller 0)[3];

  &logTrace ( 20209, "Subroutine <%s> called with:
  User                     [%s]
  WLSUser		   [%s]
  WLSPassword 	           [%s]
  nmUser		   [%s]
  nmPassword 	           [%s]
  Host                     [%s]
  Port                     [%s]
  ServerRole               [%s]
  DomainName               [%s]
  DomainDir                [%s] 
  BEA_HOME                 [%s]
  WlstScript               [%s]
  nmType                   [%s]
  AdminURL                 [%s]
  TimeOut                  [%s]",
  $sMyName,$sUser, $sWLSUser, $sWLSPassword, $snmUser, $snmPassword, $sHost, $sPort, 
  $sServerRole, $sDomainName, $sDomainDir, $sBEA_HOME, $sWlstScript, $snmType, $sAdminURL, $iTimeOut);

  my $sResourceType   = $ENV{VCS_LOG_AGENT_NAME};
  my $sResourceName   = $ENV{'VCS_LOG_RESOURCE_NAME'};
  my $sEntryPointName = $ENV{'VCS_LOG_SCRIPT_NAME'};
  my $sWLSOutFile     = "/tmp/.VRTS$sResourceType/$sResourceName"."_$sEntryPointName.slm.out";
  
  my $sProtocol		  = "t3";
  
  if ( $sAdminURL )
  {
    ($sProtocol) = $sAdminURL =~ /(\w+):\/.*/ ;
    &logTrace(20307,"<%s>:Protocol extracted from AdminURL is [%s].",$sMyName, $sProtocol);
  }

  #-------------------------------------------
  # Unlink the output file before running SLM.
  #-------------------------------------------
  unlink $sWLSOutFile;

  my ( $sName      ,
       $sPassword  ,
       $iUid       ,
       $iGid       ,
       $iQuota     ,
       $sComment   ,
       $sGcos      ,
       $sHome      ,
       $sUserShell ,
       $sExpire    ) = getpwnam($sUser);

  my $sCommand       = $sWlstScript;

  if ( ! -f $sCommand )
  {
    &logTrace(20210,"<%s>:Possibly weblogic 7/8 environment.",$sMyName);
    return( WLS78SecondLevelMonitor(@_));
  }

  my @sCommandArgsList;
  my $sCommandArgsForDisplay;

  my $sDecryptedPassword;
  if ( "NodeManager" eq $sServerRole ) {
     $sDecryptedPassword = DecryptPassword($snmPassword);
  }
  else {
     $sDecryptedPassword = DecryptPassword($sWLSPassword);
  }

  if ( ! $sDecryptedPassword )
  {
    &logError(20211,"<%s>:Could not decrypt the password",$sMyName);
    return 0;
  }

  if ( "NodeManager" eq $sServerRole )
  {
    #-------------------------------------------
    # ServerRole is NodeManager
    # Use nmConnect()
    #-------------------------------------------

    @sCommandArgsList = (
                         "nmConnect\(\"$snmUser\",\"$sDecryptedPassword\",\"$sHost\",\"$sPort\",\"$sDomainName\",\"$sDomainDir\",\"$snmType\"\)\n",
                         "dumpStack\(\)\n",
                         "exit\(\)\n" 
                        );

    $sCommandArgsForDisplay = "<< _EOF_ 
nmConnect(\"$snmUser\",XXXXXX,\"$sHost\",\"$sPort\",\"$sDomainName\",\"$sDomainDir\",\"$snmType\")
dumpStack()
exit()
_EOF_
";
  }
  else
  {
    #-------------------------------------------
    # ServerRole is Administrative or Managed.
    # Use connect()
    #-------------------------------------------

    @sCommandArgsList = (
                          "connect\(\"$sWLSUser\",\"$sDecryptedPassword\",\"${sProtocol}://$sHost:$sPort\"\)\n",
                          "dumpStack\(\)\n",
                          "exit\(\)\n"
                        );

    $sCommandArgsForDisplay = "<< _EOF_ 
connect(\"$sWLSUser\",XXXXXX,\"${sProtocol}://$sHost:$sPort\")
dumpStack()
exit()
_EOF_
";
  }

  &logTrace( 20212, "<%s>:Command Arguments [%s]",
                                  $sMyName , $sCommandArgsForDisplay);

  $iTimeOut -= 10;
  my $sEnvFile       = "";
  my $bTimedOut;
  my $iExitCode;
  my @lsCommandOut;


  #-----------------------------------------------------------------
  # Set LANG=C for parsing the output during second level monitoring
  #-----------------------------------------------------------------
  my $sEnv_before = $ENV{LANG};
  $ENV{LANG} = "C" ;

  my $iReturnCode = RunCmdViaPipe ($sUser,
                                   $sCommand,
                                   $sWLSOutFile,
                                   \@sCommandArgsList,
                                   $sBEA_HOME,
                                   $sWlstScript, 
                                   \$iExitCode,
                                   $iTimeOut,
                                   \$bTimedOut);

  #-----------------------------------
  # Re-set LANG to the original value
  #-----------------------------------
  defined $sEnv_before ? $ENV { LANG } = $sEnv_before:delete $ENV { LANG };

  if ( $iReturnCode )
  {
    #----------------------------------
    # Command executed.
    #----------------------------------
    if ( $iExitCode )
    {
      #-----------------
      # Command failed.
      #-----------------
      &logError (20213,"<%s>:[%s] failed with exit code [%s]",
                     $sMyName,$sCommand,$iExitCode);
      &logError (20214,"<%s>: bTimedOut is [%s]", $sMyName, $bTimedOut);
      if ( $bTimedOut )
      {
        &logTrace( 20215,"<%s>:Sleeping till _had kills the entry point.",$sMyName);
        sleep (1000) while(1);
      }
      return VCSDef_MonitorExit::UNKNOWN;
    }
    else
    {
      #---------------------------------
      # Command executed successfully.
      #---------------------------------
      &logTrace (20216, "<%s>:Command [%s] executed successfully.",
                 $sMyName, $sCommand);

      if (open ( FH , "$sWLSOutFile" ) )
      {
        while (<FH>)
        {
          if( /Successfully connected/i )
          {
            &logTrace(20217,"<%s>:wlst.sh reports server running.",$sMyName);
            close FH;
            return VCSDef_MonitorExit::ONLINE;
          }
        }
        &logTrace(20218,"<%s>:wlst.sh reports server NOT running.",$sMyName);
      }
      else 
      {
        &logError(20219,"<%s>:Failed to open SLM logfile [%s]",
                        $sMyName,$sWLSOutFile);
        return VCSDef_MonitorExit::UNKNOWN;
      }

      &logTrace(20220,"<%s>: wlst.sh reports server not running.",$sMyName);
      return VCSDef_MonitorExit::OFFLINE;
    }
  }
  else
  {
    #-----------------------------------
    # Problem during command execution.
    #-----------------------------------
    if ( $bTimedOut )
    {
      #--------------------
      # Command timed out.
      #--------------------
      &logError (20221,"<%s>:[%s] did not complete within [%s] seconds.",
                 $sMyName,$sCommand,$iTimeOut);
      &logTrace( 20222,"<%s>:Sleeping till had kills the entry point.",$sMyName);
      sleep (1000) while(1);
    }
    else
    {
      #--------------------
      # Command failed.
      #--------------------
      &logError (20223,"<%s>:[%s] failed.", $sMyName,$sCommand);
    }
    return VCSDef_MonitorExit::UNKNOWN;
  }
  return VCSDef_MonitorExit::ONLINE;
}

sub WLS78SecondLevelMonitor
{
  my ( $sUser,
       $sWLSUser,
       $sWLSPassword,
       $sHost,
       $sPort,
       $sServerRole,
       $sDomainName,
       $sDomainDir,  
       $sBEA_HOME,
       $sWlstScript,
       $snmType,
       $sAdminURL,
       $iTimeOut) = @_;

  my $sMyName = ( caller 0)[3];

  &logTrace ( 20224, "Subroutine <%s> called with:
  User                     [%s]
  (nm)WLSUser              [%s]
  (nm)WLSPassword          [%s]
  Host                     [%s]
  Port                     [%s]
  ServerRole               [%s]
  DomainName               [%s]
  DomainDir                [%s] 
  BEA_HOME                 [%s]
  WlstScript                  [%s]
  nmType                   [%s]
  AdminURL                [%s]
  TimeOut                  [%s]",
  $sMyName,$sUser, $sWLSUser, $sWLSPassword, $sHost, $sPort, 
  $sServerRole, $sDomainName, $sDomainDir, $sBEA_HOME, $sWlstScript, $snmType, $sAdminURL, $iTimeOut);

  my $sResourceType   = $ENV{VCS_LOG_AGENT_NAME};
  my $sResourceName   = $ENV{'VCS_LOG_RESOURCE_NAME'};
  my $sEntryPointName = $ENV{'VCS_LOG_SCRIPT_NAME'};
  my $sWLSOutFile     = "/tmp/.VRTS$sResourceType/$sResourceName"."_$sEntryPointName.slm.out";
  
  my $sProtocol		  = "t3";
  
  if ( $sAdminURL )
  {
    ($sProtocol) = $sAdminURL =~ /(\w+):\/.*/ ;
    &logTrace(20308,"<%s>:Protocol extracted from AdminURL is [%s].",$sMyName, $sProtocol);
  }

  if ( ! -f "$sDomainDir/VRTSWebLogicConfig.properties" )
  {
    &logTrace(20225,"<%s>:VRTSWebLogicConfig.properties file does not exist in $sDomainDir",$sMyName);
  }

  if ( ! -f "$sDomainDir/VRTSWebLogicKey.properties")
  {
    &logTrace(20226,"<%s>:VRTSWebLogicKey.properties file does not exist in $sDomainDir",$sMyName);
    return VCSDef_MonitorExit::ONLINE;
  }

  # Find out where is java
  #-------------------------------------------------------------------
  # Assuming that for WLS 7/8 the WlstScript is from default Location
  # i.e $WL_HOME/common/bin/wlst.sh
  #-------------------------------------------------------------------

  my $sWlstHome = dirname($sWlstScript);
  my $sWL_HOME = "$sWlstHome/../..";

  my $sEnvFile     = "$sWL_HOME/server/bin/setWLSEnv.sh";
  my @sJavaPath = `. $sEnvFile; which java`;
  
  my $sCommand     = $sJavaPath[$#sJavaPath];
  chomp $sCommand;
  my $sCommandArgs = "weblogic.Admin -url ${sProtocol}://$sHost:$sPort -userconfigfile VRTSWebLogicConfig.properties -userkeyfile VRTSWebLogicKey.properties GETSTATE";
  my $iExitCode;
  my @lsCommandOut;
  my $bTimedOut;

  #-----------------------------------------------------------------
  # Set LANG=C for parsing the output during second level monitoring
  #-----------------------------------------------------------------
  my $sEnv_before = $ENV{LANG};
  $ENV{LANG} = "C" ;

  my $iReturnCode = Sys::RunWithEnvCmdWithOutputWithTimeOut($sEnvFile,
                                                           0,                #source user env
                                                           $sCommand,
                                                           $sCommandArgs,
                                                           \$iExitCode,
                                                           $sUser,
                                                           \@lsCommandOut,
                                                           \$bTimedOut,
                                                           $iTimeOut,
                                                           "",
                                                           $sDomainDir);
  #-----------------------------------
  # Re-set LANG to the original value
  #-----------------------------------
  defined $sEnv_before ? $ENV { LANG } = $sEnv_before:delete $ENV { LANG };

  if ( $iReturnCode )
  {
    #---------------------------
    # Command got executed.
    #---------------------------
    if ( $iExitCode )
    {
      #---------------------
      # Command failed !
      #---------------------
      &logTrace (20227,"<%s>:Command [%s] with args [%s] failed with exit code [%s]",
                 $sMyName,$sCommand,$sCommandArgs,$iExitCode);
      return;
    }
    else
    {
      #------------------------
      # Successful execution.
      #------------------------
      &logTrace(20228,"<%s>:Command [%s] with args [%s] successfully executed with exit code [%s]",
                           $sMyName,$sCommand,$sCommandArgs,$iExitCode);
      foreach my $sLine (@lsCommandOut )
      {
        if ( $sLine =~ /RUNNING/ )
        {
          &logTrace(20229,"<%s>:Found pattern 'RUNNING' for this server.",$sMyName);
          return VCSDef_MonitorExit::ONLINE;
        }
      }
      &logError(20230,"<%s>:Did not find pattern 'RUNNING' for this server.",$sMyName);
    }
  }
  else
  {
    if ( $bTimedOut )
    {
      #--------------------
      # Command timed out.
      #--------------------
      &logError (20231,"<%s>:Command [%s] with args [%s] did not complete within [%s] seconds.",
      $sMyName,$sCommand,$sCommandArgs,$iTimeOut);
      &logTrace( 20232,"<%s>:Sleeping till had kills the entry point.",$sMyName);
      sleep (1000) while(1);
    }
    else
    {
      #--------------------
      # Unknown error.
      #--------------------
      &logError(20233,"<%s>:Command [%s] with args [%s] execution is unknown.",
                       $sMyName,$sCommand,$sCommandArgs);
    }
  }

  return VCSDef_MonitorExit::OFFLINE;
}


=pod

 -------------------------------------------------------------------------
 Function : int RunSecondLevelMonitor( IN Integer SecondLevelMonitor )

 Description :  Determines whether now is the time to run the Second
 Level Monitoring checks for the WebLogic resource. This subroutine 
 also maintains the slm counter file at the location :
 /tmp/.VRTS<ResourceType>/<ResourceName>.slm
 eg. /tmp/.VRTSWebLogic/WLS90Mng01_nodemanager.slm
 The file is updated every monitor cycle if SecondLevelMonitor is value
 is greater than 1. 

 Parameters :

  $iSecondLevelMonitor    SecondLevelMonitor attribute value

 Returns : Integer values as defined below.

  0  Second Level Monitor should not be run.
  1  Second Level Monitor need to be run.

 -------------------------------------------------------------------------

=cut

sub RunSecondLevelMonitor 
{
  #----------------
  # Get Arguments..
  #----------------
  my ( $iSecondLevelMonitor) = @_;
       
  my $sResourceType   = $ENV{VCS_LOG_AGENT_NAME};
  my $sResourceName   = $ENV{'VCS_LOG_RESOURCE_NAME'};

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20234, "Subroutine <%s> called:
  SecondLevelMonitor        [%s]
  ResourceType              [%s]
  ResourceName              [%s]",
  $sMyName, $iSecondLevelMonitor, $sResourceType, $sResourceName );

  #---------------------------
  # Set Return Code to false..
  #---------------------------
  my $iReturnCode = 0; # Do not run SLM..

  #------------------------
  # Setup local variables..
  #------------------------
  my $sSLMDir  = "/tmp/.VRTS$sResourceType";
  my $sSLMFile = "$sSLMDir/$sResourceName.slm";
  my $iSLMCntr = undef;

  #--------------------------------------------
  # Validate SecondLevelMonitor trivial cases..
  #--------------------------------------------
  if ( 0 == $iSecondLevelMonitor ||
       1 == $iSecondLevelMonitor ) 
  {
    $iReturnCode = $iSecondLevelMonitor; # Assign..

    &logTrace (20235, "<%s>:Removing SLM file [%s] if it exists. Returning with [%s]", $sMyName, $sSLMFile, $iSecondLevelMonitor );

    unlink $sSLMFile if ( -e $sSLMFile ); # Remove SLM file..

    return $iReturnCode;
  }

  #----------------------------------
  # Create history folder if needed..
  #----------------------------------
  if ( -d $sSLMDir ) 
  {
    &logTrace ( 20236, "<%s>:SLM folder [%s] exists", $sMyName, $sSLMDir );
  }
  else 
  {
    mkdir ( $sSLMDir, 0777 ) ||
    &logWarn ( 20237, "<%s>:Failed to create SLM folder [%s]",
    $sMyName, $sSLMDir );
    chmod (0777,$sSLMDir );
    return 1; # Run SLM..
  }
  
  #--------------------------
  # Create SLM history file..
  #--------------------------
  if ( sysopen ( SLMFILE, $sSLMFile, O_RDWR|O_SYNC|O_CREAT, 0600 ) ) {
    &logTrace ( 20238,
    "<%s>:Opened SLM file [%s] successfully for read/write",
    $sMyName, $sSLMFile );
  }
  else {
    &logWarn ( 20239,
    "<%s>:Failed to open SLM file [%s] for read/write",
    $sMyName, $sSLMFile );
    return 1; # Run SLM..
  }
  
  #---------------
  # Read in file..
  #---------------
  ( $iSLMCntr ) = grep { s/\n// } <SLMFILE>; 

  #-----------------
  # Corrupted file..
  #-----------------
  if ( ! defined $iSLMCntr || $iSLMCntr < 0 ) {

    $iSLMCntr ||= ''; # Avoid undefs..

    &logTrace ( 20240,
    "<%s>:SLM file [%s] contains [%s]. Re-setting to [%s]",
    $sMyName, $sSLMFile, $iSLMCntr, $iSecondLevelMonitor );
    $iSLMCntr = $iSecondLevelMonitor; # Set to default..
  }
  
  #-----------------------------------------
  # Update counter, if less than requested..
  #-----------------------------------------
  if ( $iSecondLevelMonitor > $iSLMCntr ) {
    ++ $iSLMCntr;
  }
  else {
    $iSLMCntr = 1;    # Now's the time..
    $iReturnCode = 1; # Run SLM..
  }
  
  #------------------
  # Update the file..
  #------------------
  seek ( SLMFILE, 0, 0 ); # Rewind before update..

  &logTrace ( 20241,
  "<%s>:Updating SLM file [%s] with [%s]. Return code is [%s]",
  $sMyName, $sSLMFile, $iSLMCntr, $iReturnCode );

  print SLMFILE "$iSLMCntr\n"; # Update..
  
  close ( SLMFILE ) ||
  &logWarn ( 20242,
  "<%s>:Close failed on SLM file [%s] after update",
  $sMyName, $sSLMFile, $! );

  return $iReturnCode;
}

=pod

-----------------------------------------------------------------------------

Function: int WaitForAdminServer ( IN Integer ExpirationTime, 
                                   IN String  AdminUrl );

Description:
  Suspends the logical execution of the agent until the Admin Server is 
  available  and  ready  to accept  connections , or it has reached the 
  ExpirationTime  passed to it. It will test to see if the Admin Server 
  is ready every 5 seconds.  

Parameters:

  ExpirationTime   The time to exit this subroutine
  AdminUrl         The URL of the WebLogic Administrative Server. 
                   It should be formatted like:  http://wlsadmin:7001

Returns:
  0   The ExpirationTime was reached prior to the Admin Server becoming 
      ready to accept connections
  1   The Admin Server became available and ready to accept connections in 
      time.

-----------------------------------------------------------------------------

=cut

sub WaitForAdminServer
{
  my ( $iExpirationTime, $sAdminURL) = @_;

  #---------------------
  # Announce Entry...
  #---------------------
  my $sMyName = ( caller 0 )[3];
  &logTrace(20243,"Subroutine <%s> called with ExpirationTime [%s] AdminURL [%s]",
             $sMyName,$iExpirationTime, $sAdminURL);

  my @lsURLcomponents = WebLogic::ParseURL($sAdminURL);
  my ( $sAdminHost,$iAdminPort ) = ( $lsURLcomponents[1], $lsURLcomponents[2] );

  my $iNSR = $iExpirationTime - time;
  if ( $iNSR <= 5 )
  {
    &logTrace(20244,"<%s>:iNSR [%s] $iExpirationTime [%s].",$sMyName,$iNSR,$iExpirationTime);
    &logInfo(20245,"<%s>:Not enough time left to wait for the WebLogic Admin Server.",$sMyName);
    return 0;
  }

  my $iCurrentTime = time;
  while ( ( $iCurrentTime + CHECK_INTERVAL  ) < $iExpirationTime )
  {
    &logTrace( 20246, "<%s>:The current time [%s] + wait interval [%s] == [%s] is still less than the calculated OnlineTimeout expiration time [%s]",
                         $sMyName,
                         $iCurrentTime,
                        CHECK_INTERVAL,
                        $iCurrentTime + CHECK_INTERVAL ,
                        $iExpirationTime );

    if ( 1 == Comms::CheckPort($sAdminHost,$iAdminPort) )
    {
      &logInfo(20247,"<%s>:Admin server at [%s:%s] responding correctly.",
                $sMyName,$sAdminHost,$iAdminPort); 
      return 1;
    }
    &logInfo(20248,"<%s>:Admin Server at [%s:%s] is not ready. Will check again in [%s] seconds.", $sMyName,$sAdminHost,$iAdminPort,CHECK_INTERVAL);
    sleep CHECK_INTERVAL;
    $iCurrentTime = time;
  }

  &logWarn( 20249, "<%s>:Could not connect to WLS Admin Server at [%s] using port [%s] before the allowed timeout period expired, giving up",
                     $sMyName,
                     $sAdminHost,
                     $iAdminPort );

  return 0;
}

=pod

----------------------------------------------------------------------------
Function: Boolean __RunWithEnvCmdWithOutputWithTimeOut( 
                                     IN  String    Command,
                                     IN  String    CommandArgs,
                                     OUT Integer   CommandReturnCode ,
                                     IN  String    User,
                                     OUT Array     CommandOut ,
                                     OUT Boolean   TimedOut,
                                     IN  Integer   TimeoutSeconds ,
                                     IN  String    OutFileName ,
                                     IN  String    FromDir );

Description:
  This function executes the specified command by sourcing environment file
  if needed and as specified user. It executes the command with timeout value.

Parameters:
  Command           : Command the needs to be executed.
  CommandArgs       : The arguments that needs to be passed to the command.
  CommandReturnCode : The return code of the command executed.
  User              : The user name with which the command needs to be executed.
  CommandOut        : The output of the command executed.
  TimeOut           : Flag indicating whether the command failed to execute 
                      due to timeout.
  TimeoutSeconds    : Maximum time the command can execute before getting
                      signaled out due to timeout.
  OutFileName       : Output of the command will be redirected to this file,
                      only if specified. NULL value is permitted.
  FromDir           : Command will be executed after doing a chdir to this dir.
  DisplayOutput     : Boolean Command Output will be displayed if set to 1.

Return Value:
  Boolean value from the set of {0,1} where
         0 - command timedout
         1 - command not timedout.
----------------------------------------------------------------------------

=cut

sub __RunWithEnvCmdWithOutputWithTimeOut 
{
  my ( $sCommand,
       $sCommandArgs,
       $riCommandReturnCode,
       $sUser,
       $rlsCommandOut,
       $rbTimedOut,
       $iTimeoutSeconds,
       $sOutFileName,
       $sFromDir,
       $bDisplayOutput ) = @_ ;

  my $sMyName = (caller 0)[3];

  &logTrace(20250,"<%s>:Command [%s], User [%s], Timeout [%s], OutFile [%s], FromDir [%s], DisplayOutput [%s]",
                   $sMyName, $sCommand, $sUser, $iTimeoutSeconds, $sOutFileName, $sFromDir , $bDisplayOutput );

  #-----------------------------
  # Establish local variables.
  #-----------------------------
  my $sCmdToRun     = "";
  my $sCommandLine  = "";

  if( ! defined $sUser || $sUser eq "" ) {
    &logTrace(20251,"<%s>:No user is set. Setting User to root.", $sMyName );
    $sUser = "root";
  }

  my ( $sName      ,
       $sPassword  ,
       $iUid       ,
       $iGid       ,
       $iQuota     ,
       $sComment   ,
       $sGcos      ,
       $sHome      ,
       $sUserShell ,
       $sExpire    ) = getpwnam($sUser);

  my $bCshSet       = 0;
  if ( $sUserShell =~ /(.*)\/bin\/.*csh$/ )
  {
    $bCshSet = 1;
  }

  #------------------------------------
  # Confirm existance of Exectuable.
  #------------------------------------
  if ( ! -e $sCommand ) {
    &logError(20252,"<%s>:File [%s] does not exist",$sMyName, $sCommand ) ;
    return 0 ;
  }
  elsif ( ! -x $sCommand ) {
    &logError(20253,"<%s>:File [%s] is not executable",$sMyName, $sCommand ) ;
    return 0 ;
  }
  else {
    if( defined $sCommandArgs || $sCommandArgs ne "" ) {
      $sCmdToRun .= "$sCommand $sCommandArgs" ;
    }
    else {
      $sCmdToRun .= $sCommand ;
    }
  }

  if ( $sOutFileName )
  {
    if ( $bCshSet )
    {
      $sCmdToRun .= ">& $sOutFileName ";
    }
    else
    {
      $sCmdToRun .= "> $sOutFileName 2>&1";
    }
  }

  &logTrace(20254,"<%s:>Going to run command line [%s] as User [%s]", 
                        $sMyName , $sCmdToRun , $sUser );

  #---------------------------------------------------------------------
  # If the request is to run the command within the same user context as
  # this process is already running within, then we do not have to
  # do the command from under "su":
  #---------------------------------------------------------------------
  $sCommandLine = "$::gsCommandSu $sUser $::gsCommandArgsSu '$sCmdToRun'";
    
  #---------------------------------------
  # Set timeout flag to true for now.
  #---------------------------------------
  $$rbTimedOut = 1;

  if ($iTimeoutSeconds < 1 ) {
    &logTrace(20255,"<%s>:Passed in timeout is less than 1 second, defaulting it to 15 seconds.",$sMyName) ;
    $iTimeoutSeconds = 15;
  } 
  my $iChildPid = '';
  my @sCmdOut   = ();
  
  eval {
    local $SIG{ALRM} = sub {
      &logTrace(20256,"<%s>:Command timed out, going to attempt to kill the blocking child processes",$sMyName);
      if( defined $iChildPid && $iChildPid > 1 ) {
        Proc::KillPIDandAllChildren( $iChildPid );
      }
      else {
        &logTrace(20257,"<%s>:The Child PID was either undefined or less than 1.  Therefore we cannot kill the blocking child process",$sMyName );
      }
      die "SIGALARM JUMP EVAL";
    };
    alarm $iTimeoutSeconds;

    my $sFullCmd = "$sCommandLine 2>&1";
    $iChildPid = open(CMD, "$sFullCmd |")
         or &logError(20258, "<%s>:Can't fork a process: [%s]", $sMyName, $! ) ;
    if( defined $iChildPid && $iChildPid > 1 ) {
      @sCmdOut = <CMD>;
    }
    else {
      &logError(20259,"<%s>:The attempt to open a pipe to command resulted in an invalid child PID of [%s]",
                       $sMyName , $iChildPid );
    }

    my $sLastOS_Error = $!;
    if( close(CMD) ) {
      $$riCommandReturnCode = $? >> 8;
    }
    else {
      $$riCommandReturnCode = $? >> 8;
      #-----------------------------------------------------------------------
      # Commands that exit non-zero, which sometimes does not indicate that
      # it failed, causes the close to return zero instead of >= 1.
      #-----------------------------------------------------------------------
      &logTrace(20260,"<%s>:Command line provided a non-zero exit code -- this does not necessarily indicate a problem... (Perl's OS error variable prior to the command-pipe close was [%s], and after the close was [%s] )",
                       $sMyName , $sLastOS_Error, $! );
    }

    @$rlsCommandOut = @sCmdOut;

    #---------------------
    # cancel the alarm
    #---------------------
    alarm 0;

    #---------------------------
    # command didn't timeout.
    #---------------------------
    $$rbTimedOut = 0; 
  };

  if ( $$rbTimedOut ) {
    &logError(20261,"<%s>:The command did not complete within [%s] seconds",
                     $sMyName , $iTimeoutSeconds );
    
    return 0; # return invalid as the script timed out.
  }
  else 
  {
    return 1;
  }
}


=pod

----------------------------------------------------------------------------
Function:  String DecryptPassword ( IN String CryptedPassword)

Decrypts the password using VCS decrypt subroutine.

----------------------------------------------------------------------------

=cut

sub DecryptPassword
{
  my ($sCryptedPassword ) = @_;

  my $sMyName = ( caller 0 )[3];
  &logTrace(20262,"Subroutine <%s> called.", $sMyName );

  my $sDecryptCmd = $HADECRYPT ||= "/opt/VRTSvcs/sbin/vcsdecrypt";

  #-----------------------------------------------------------------
  # Check decrypt command and return immediately if not executable.
  #-----------------------------------------------------------------
  if ( ! -x $sDecryptCmd )
  {
    &logError( 20263, "<%s>:Decrypt command [%s] is not an executable.",
                       $sMyName, $sDecryptCmd);
    return 0;
  }

  #----------------------------------------
  # Execute the actual vcsdecrypt command.
  #----------------------------------------
  my $iExitCode;
  my @lsCommandOut;
  my $bTimedOut,
  my $iTimeOut = 5;
  my $iReturnCode;
  my $bDisplayOutput = 0;  #Dont display the password

  $iReturnCode = WebLogic::__RunWithEnvCmdWithOutputWithTimeOut(
                                                  $sDecryptCmd,
                                                  $sCryptedPassword,
                                                  \$iExitCode,
                                                  "root",
                                                  \@lsCommandOut,
                                                  \$bTimedOut,
                                                  $iTimeOut,
                                                  "",
                                                  "",
                                                  $bDisplayOutput);
  if ( $iReturnCode )
  {
    #----------------------------------
    # Command executed.
    #----------------------------------
    if ( $iExitCode )
    {
      #-----------------
      # Command failed.
      #-----------------
      &logError (20264,"<%s>:[%s] failed with exit code [%s]",
                     $sMyName,$sDecryptCmd,$iExitCode);
      return ;
    }
    else
    {
      #---------------------------------
      # Command executed successfully.
      #---------------------------------
      &logTrace (20265, "<%s>:Command [%s] executed successfully.",
                 $sMyName, $sDecryptCmd);
      return $lsCommandOut[0] ;
    }
  }
  else
  {
    #-----------------------------------
    # Problem during command execution.
    #-----------------------------------
    if ( $bTimedOut )
    {
      #--------------------
      # Command timed out.
      #--------------------
      &logError (20266,"<%s>:[%s] did not complete within [%s] seconds.",
                 $sMyName,$sDecryptCmd,$iTimeOut);
      return;
    }
    else
    {
      #--------------------
      # Command failed.
      #--------------------
      &logError (20267,"<%s>:[%s] failed.", $sMyName,$sDecryptCmd);
    }
    return;
  } 
}


=pod

----------------------------------------------------------------------------
Function: Boolean RunCmdViaPipe( IN  String    User,
                                 IN  String    Command,
                                 IN  String    OutFile,
                                 IN  String    CommandArgs,
                                 IN  String    FromDir,
                                 IN  String    WlstScript,
                                 OUT Integer   CommandReturnCode ,
                                 IN  Integer   TimeoutSeconds,
                                 OUT Boolean   TimedOut);

Description:
  This function executes the specified command in an interactive program
  via pipe mechanism as  specified  user. It executes   the command with 
  timeout value. The output is dup'd in the OutFile specified.

Parameters:
  User              : The user name with which the command needs to be executed.
  Command           : Command the needs to be executed.
  OutFileName       : Output of the command will be redirected to this file.
  CommandArgs       : The arguments that needs to be passed to the command.
                       Reference to list of commands. Each member of the list
                       is an eol'd string.
  FromDir           : Command will be executed after doing a chdir to this dir.
  CommandReturnCode : The return code of the command executed.
  TimeoutSeconds    : Maximum time the command can execute before getting
                      signaled out due to timeout.
  TimeOut           : Flag indicating whether the command failed to execute 
                      due to timeout.

Return Value:
  Boolean value from the set of {0,1} where
         0 - command timedout
         1 - command not timedout.
----------------------------------------------------------------------------

=cut

sub RunCmdViaPipe
{
  my ( $sUser,
       $sCommand ,
       $sOutFile,
       $rsCommandArgs ,
       $sBEA_HOME,
       $sWlstScript,
       $riCommandReturnCode ,
       $iTimeoutSeconds,
       $rbTimedOut ) = @_;

  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20268, "Subroutine <%s> called with:
  User                     [%s]
  Command                  [%s]
  OutFileString            [%s]
  BEA_HOME                 [%s]
  WlstScript                  [%s] 
  CommandReturnCode        [%s]
  TimeoutSeconds           [%s]",
  $sMyName, $sUser, $sCommand, $sOutFile, $sBEA_HOME, $sWlstScript, $$riCommandReturnCode, $iTimeoutSeconds);

  if ( ! -x $sCommand )
  {
    &logError(20269,"<%s>:[$sCommand] is not executable. Check file existence and permissions.",$sMyName, $sCommand);
    return 0;
  }

  $$rbTimedOut = 1;
  my $iChildPid = open( CMD, "|-");

  $SIG{PIPE} = sub { 
                     &logError(20270 , 
                               "<%s>:Broken pipe for [%s]" , 
                               $sMyName, $sCommand);
                     return 0;
                   };

  if ( $iChildPid ) 
  {
    #------------------
    # Parent process.
    #------------------
    $$rbTimedOut = 0;
    eval 
    {
      local $SIG{ALRM} = sub 
      {
        #---------------------------
        # Alarm handled for timeout
        #---------------------------
        &logInfo(20271,"<%s>:Command [%s] timed out, going to attempt to kill the blocking child processes",
                 $sMyName, $sCommand);
        $$rbTimedOut = 1;
        if( defined $iChildPid && $iChildPid > 1 ) 
        {
          Proc::KillPIDandAllChildren( $iChildPid );
        }
        else 
        {
          &logInfo(20272,"<%s>:The Child PID was either undefined or less than 1. Therefore we cannot kill the blocking child process",$sMyName);
        }
        return 0;
      };

      #------------------
      # Set the timeout
      #------------------
      alarm $iTimeoutSeconds;

      for (@$rsCommandArgs)
      {
        #--------------------------------
        # Write the commands to the pipe
        #--------------------------------
        print CMD;
      }
      if ( close( CMD ) )
      {
        #--------------------------------
        # Command execution was good.
        #--------------------------------
        &logTrace(20273,"<%s>:[%s] Command executed successfully.", $sMyName, $sCommand);
        $$riCommandReturnCode = $? ;
      }
      else
      {
        #------------------------------------------
        # Command execution returned some code.
        #------------------------------------------
        $$riCommandReturnCode = $? ;
        &logError(20274,"<%s>:Command [%s] execution was not successful.[%s]",
                  $sMyName , $sCommand , $?);
      }
      #---------------------
      # cancel the alarm
      #---------------------
      alarm 0;

      #---------------------------
      # command didn't timeout.
      #---------------------------

    }; #end of eval
  }
  elsif ( 0 == $iChildPid ) 
  {
    #------------------
    # Child process.
    #------------------
    my ( $sName      ,
         $sPassword  ,
         $iUid       ,
         $iGid       ,
         $iQuota     ,
         $sComment   ,
         $sGcos      ,
         $sHome      ,
         $sUserShell ,
         $sExpire    ) = getpwnam($sUser);

    chdir ( $sBEA_HOME );
    #------------------------------------------
    # SetGid and Setuid of the User attribute
    #------------------------------------------
    POSIX::setgid($iGid);
   
    #--------------
    # Get File Gid
    #--------------
   
    my ( $wlstFile, $iFileGid ) = 0;

    $wlstFile=$sWlstScript;
    $iFileGid = ( stat $wlstFile ) [5];

    &logTrace(20275,"<%s>: wlst.sh File's Group owner is [%s].", $sMyName, $iFileGid);
 
    #----------------------
    # Get members of $fGid
    #----------------------

    my ( $sFileGrpName, $lnGrpMembers, 
         @lsGrpMembers, $iMemberUid, $sFileUser ) = ();

    $sFileGrpName = getgrgid ( $iFileGid );
    $lnGrpMembers = ( getgrnam $sFileGrpName ) [3];
    @lsGrpMembers = split (' ', $lnGrpMembers ); 

    &logTrace(20276,"<%s>: wlst.sh File's Group members are [%s].", $sMyName, $lnGrpMembers);

    #------------------------------
    # Set $iUid is member of $fGid
    #------------------------------
    foreach $sFileUser ( @lsGrpMembers ){
        $iMemberUid = getpwnam ( $sFileUser );
        if ( $iMemberUid && $iMemberUid == $iUid ){
           $) = "$iGid $iFileGid $)";
           &logTrace(20277,"<%s>:Set Gid to [%s].", $sMyName, $iGid);
        }
    }
    &logTrace(20278,"<%s>:Set Uid to [%s].", $sMyName, $iMemberUid);
    POSIX::setuid($iUid);

    #----------------------------------------------
    # Dup the stdout and stderr of this child to 
    # be exec'd to the OutFile
    #----------------------------------------------
    open STDOUT, '>', "$sOutFile" or 
    die "<$sMyName>:Can't redirect STDOUT to $sOutFile: $! ";
    open STDERR, ">&STDOUT" or 
    die "<$sMyName>:Can't redirect STDERR to $sOutFile: $!";

    #--------------
    # Exec Command
    #--------------
    exec("$sCommand ") || 
    &logError(20279,"<%s>:Cannot execute [%s]",$sMyName, $sCommand);
    return 0;
  }
  else 
  {
    #------------------------
    # Cannot fork process !
    #------------------------
    &logError(20280,"<%s>:Cannot fork [%s]",$sMyName, $sCommand);
    return 0;
  }

  if ( $$rbTimedOut ) 
  {
    &logError(20281,"<%s>:The command did not complete within [%s] seconds", 
                    $sMyName, $iTimeoutSeconds );
    return 0; # return invalid as the script timed out.
  } 
  else 
  {
    return 1;
  }
}

1; # So PERL knows its included..
